(*
  Formats.m

  Formatting commands.

   ----------------------------------------------------------------- *)

BeginPackage["LinksGouldExplorer`"];

(* Begin["`Private`"]; *)

Unprotect[KroneckerDelta];
KroneckerDelta /:
  MakeBoxes[KroneckerDelta[a_, b_], StandardForm] :=
    SubsuperscriptBox[
      "\[Delta]", MakeBoxes[a, StandardForm], MakeBoxes[b, StandardForm]
    ];
Protect[KroneckerDelta];

Uqgl /:
  MakeBoxes[Uqgl[m_, n_], StandardForm] :=
    RowBox[{
      SubscriptBox["U", "q"],
      MakeBoxes["[gl(", StandardForm],
      MakeBoxes[m, StandardForm], "|", MakeBoxes[n, StandardForm],
      MakeBoxes[")]", StandardForm]
    }];

Unprotect[NonCommutativeMultiply];

  NonCommutativeMultiply /:
    MakeBoxes[NonCommutativeMultiply[WW_, Y_], StandardForm] :=
      RowBox[{
        MakeBoxes[WW, StandardForm],
        MakeBoxes[Y, StandardForm]
      }];

Protect[NonCommutativeMultiply];

NormalOrder /:
  MakeBoxes[NormalOrder[Uqgl[m_, n_], WW_], StandardForm] :=
  RowBox[{
    MakeBoxes["NormalOrder[", StandardForm],
    MakeBoxes[WW, StandardForm],
    MakeBoxes["]", StandardForm]
  }];

(* ----------------------------------------------------------------- *)

GradingOfAlgebraIndex /:
  MakeBoxes[GradingOfAlgebraIndex[Algebra_, a_], StandardForm] :=
  RowBox[{
    MakeBoxes["[", StandardForm],
    MakeBoxes[a, StandardForm],
    MakeBoxes["]", StandardForm]
  }];

(* ----------------------------------------------------------------- *)

Generator /:
  MakeBoxes[Generator[Uqgl[m_, n_], a_, a_, 0], StandardForm] :=
    RowBox[{"Id"}];

Generator /:
  MakeBoxes[Generator[Uqgl[m_, n_], a_, a_, 1], StandardForm] :=
    SubscriptBox["K", MakeBoxes[a, StandardForm]];

Generator /:
  MakeBoxes[Generator[Uqgl[m_, n_], a_, a_, +1], StandardForm] :=
    SubscriptBox["K", MakeBoxes[a, StandardForm]];

Generator /:
  MakeBoxes[Generator[Uqgl[m_, n_], a_, a_, -1], StandardForm] :=
    SubscriptBox[
      MakeBoxes[OverBar["K"], StandardForm],
      MakeBoxes[a, StandardForm]
    ];

Generator /:
  MakeBoxes[Generator[Uqgl[m_, n_], a_, a_, N_], StandardForm] :=
    SuperscriptBox[
      SubscriptBox["K", MakeBoxes[a, StandardForm]],
      MakeBoxes[StyleForm[N, FontSize->6], StandardForm]
    ] /; (N > 1);

Generator /:
  MakeBoxes[Generator[Uqgl[m_, n_], a_, a_, N_], StandardForm] :=
    With[
      {MN = - N},
      SuperscriptBox[
        SubscriptBox[
          MakeBoxes[OverBar["K"], StandardForm],
          MakeBoxes[a, StandardForm]],
        MakeBoxes[StyleForm[MN, FontSize->6], StandardForm]
      ]
    ] /; (N < -1);

Generator /:
  MakeBoxes[Generator[Uqgl[m_, n_], a_, a_, N_], StandardForm] :=
    SuperscriptBox[
      SubscriptBox[
        MakeBoxes["K", StandardForm],
        MakeBoxes[a, StandardForm]],
      MakeBoxes[StyleForm[N, FontSize->6], StandardForm]
    ];

Generator /:
  MakeBoxes[Generator[Uqgl[m_, n_], a_, b_, 1], StandardForm] :=
    SubscriptBox[
      SuperscriptBox["E", MakeBoxes[a, StandardForm]],
      MakeBoxes[b, StandardForm]
    ];

Generator /:
  MakeBoxes[Generator[Uqgl[m_, n_], a_, b_, N_], StandardForm] :=
    SuperscriptBox[
      RowBox[{
        MakeBoxes["(", StandardForm],
        SubscriptBox[
          SuperscriptBox["E", MakeBoxes[a, StandardForm]],
          MakeBoxes[b, StandardForm]
        ],
        MakeBoxes[")", StandardForm]
      }],
      MakeBoxes[N, StandardForm]
    ] /; (N =!= 1)

Generator /:
  MakeBoxes[Generator[gl[m_, n_], a_, b_], StandardForm] :=
    SubscriptBox[
      SuperscriptBox["e", MakeBoxes[a, StandardForm]],
      MakeBoxes[b, StandardForm]
    ];

AltGenerator /:
  MakeBoxes[AltGenerator[Uqgl[m_, n_], a_, b_, 1], StandardForm] :=
    SubscriptBox[
      SuperscriptBox[
        StyleBox["E", FontWeight -> Bold],
        MakeBoxes[a, StandardForm]
      ],
      MakeBoxes[b, StandardForm]
    ];

AltGenerator /:
  MakeBoxes[AltGenerator[Uqgl[m_, n_], a_, b_, N_], StandardForm] :=
    SuperscriptBox[
      RowBox[{
        MakeBoxes["(", StandardForm],
        SubscriptBox[
          SuperscriptBox[
            StyleBox["E", FontWeight -> Bold],
          MakeBoxes[a, StandardForm]],
          MakeBoxes[b, StandardForm]
        ],
        MakeBoxes[")", StandardForm]
      }],
      MakeBoxes[N, StandardForm]
    ] /; (N =!= 1)

(* ----------------------------------------------------------------- *)

CoProduct /:
  MakeBoxes[CoProduct[WW_], StandardForm] :=
    RowBox[{
      \[CapitalDelta], "(", MakeBoxes[WW, StandardForm], ")"
    }];

CoUnit /:
  MakeBoxes[CoUnit[WW_], StandardForm] :=
    RowBox[{
      \[Epsilon], "(", MakeBoxes[WW, StandardForm], ")"
    }];

Antipode /:
  MakeBoxes[Antipode[WW_], StandardForm] :=
    RowBox[{
      "S", "(", MakeBoxes[WW, StandardForm], ")"
    }];

(* ----------------------------------------------------------------- *)

alpha /:
  MakeBoxes[alpha, StandardForm] :=
    MakeBoxes[\[Alpha]];

omega /:
  MakeBoxes[omega, StandardForm] :=
    MakeBoxes[\[Omega]];

LGIHighestWeight /:
  MakeBoxes[LGIHighestWeight[Uqgl[m_, n_]], StandardForm] :=
    MakeBoxes[\[CapitalLambda], StandardForm];

qSub /:
  MakeBoxes[qSub[Uqgl[m_, n_], a_], StandardForm] :=
    SubscriptBox["q", MakeBoxes[a, StandardForm]];

DeltaSub /:
  MakeBoxes[DeltaSub[Uqgl[m_, n_], a_], StandardForm] :=
    RowBox[{
      SubscriptBox["q", MakeBoxes[a, StandardForm]],
      MakeBoxes["-", StandardForm],
      SubsuperscriptBox[
        "q", MakeBoxes[a, StandardForm], MakeBoxes[-1, StandardForm]
      ]
    }];

(* ----------------------------------------------------------------- *)

ket /:
  MakeBoxes[ket[i_], StandardForm] :=
    RowBox[{
      MakeBoxes["|", StandardForm],
      MakeBoxes[i, StandardForm],
      MakeBoxes[">", StandardForm]
    }];

bra /:
  MakeBoxes[bra[i_], StandardForm] :=
    RowBox[{
      MakeBoxes["<", StandardForm],
      MakeBoxes[i, StandardForm],
      MakeBoxes["|", StandardForm]
    }];

(* ----------------------------------------------------------------- *)

e /:
  MakeBoxes[e[{}, li_], StandardForm] :=
    SubscriptBox[
      "e",
      RowBox[Map[MakeBoxes[#, StandardForm]&, li]]
    ];

e /:
  MakeBoxes[e[ui_, {}], StandardForm] :=
    SuperscriptBox[
      "e",
      RowBox[Map[MakeBoxes[#, StandardForm]&, ui]]
    ];

(* ----------------------------------------------------------------- *)

PrimeKnot /:
  MakeBoxes[PrimeKnot[c_, alt_, i_], StandardForm] :=
    SubsuperscriptBox[
      c,
      MakeBoxes[i, StandardForm],
      MakeBoxes[alt, StandardForm]
    ];

AlexanderBriggsPrimeKnot /:
  MakeBoxes[AlexanderBriggsPrimeKnot[c_, i_], StandardForm] :=
    SubscriptBox[
      c,
      MakeBoxes[i, StandardForm]
    ];

(* ----------------------------------------------------------------- *)

theta /:
  MakeBoxes[theta[i_Integer], StandardForm] :=
    SubscriptBox[\[Theta], MakeBoxes[i, StandardForm]];

(* ----------------------------------------------------------------- *)

qBracket /:
  MakeBoxes[qBracket[x_], StandardForm] :=
    RowBox[{
      MakeBoxes["[", StandardForm],
      MakeBoxes[x, StandardForm],
      SubscriptBox[
        MakeBoxes["]", StandardForm],
        MakeBoxes["q", StandardForm]
      ]
    }];

Del /:
  MakeBoxes[Del, StandardForm] :=
    MakeBoxes[\[CapitalDelta]];

Nab /:
  MakeBoxes[Nab, StandardForm] :=
    MakeBoxes[\[EmptyDownTriangle]];

A /:
  MakeBoxes[A[j_Integer], StandardForm] :=
    SubscriptBox["A", MakeBoxes[j, StandardForm]];

A /:
  MakeBoxes[A[i_Integer, j_Integer], StandardForm] :=
    SubscriptBox[
      "A",
      RowBox[{
        MakeBoxes[i, StandardForm], ",",
        MakeBoxes[j, StandardForm]
      }]
    ];

CC /:
  MakeBoxes[CC[i_Integer, j_Integer], StandardForm] :=
    SubscriptBox[
      "C",
      RowBox[{MakeBoxes[i, StandardForm], MakeBoxes[j, StandardForm]}]
    ];

CC /:
  MakeBoxes[CC[j_Integer], StandardForm] :=
    SubscriptBox["C", MakeBoxes[j, StandardForm]];

U /:
  MakeBoxes[U[i_Integer], StandardForm] :=
    SubscriptBox["U", MakeBoxes[i, StandardForm]];

S /:
  MakeBoxes[S[i_Integer, exp_Integer], StandardForm] :=
    With[
      {
        sup = If[exp==1, "+", "-"]
      },
      SubsuperscriptBox[
        "S", MakeBoxes[i, StandardForm], MakeBoxes[sup, StandardForm]
      ]
    ];

(* ----------------------------------------------------------------- *)

TensorProduct /:
  MakeBoxes[TensorProduct[ket[i_], ket[j_]], StandardForm] :=
    RowBox[{
      "|", MakeBoxes[i, StandardForm], "\[CircleTimes]",
      MakeBoxes[j, StandardForm], ">"
    }];

TensorProduct /:
  MakeBoxes[TensorProduct[bra[i_], bra[j_]], StandardForm] :=
    RowBox[{
      "<", MakeBoxes[i, StandardForm], "\[CircleTimes]",
      MakeBoxes[j, StandardForm], "|"
    }];

TensorProduct /:
  MakeBoxes[TensorProduct[WW_, Y_], StandardForm] :=
    RowBox[{
      MakeBoxes[WW, StandardForm],
      "\[CircleTimes]",
      MakeBoxes[Y, StandardForm]
    }];

(* ----------------------------------------------------------------- *)

Unprotect[Power];
  Format[Power[WW_, 1/2]] :=
    WW Superscript[SequenceForm["1", "/", "2"]];
Protect[Power];

(* ----------------------------------------------------------------- *)

(* End[]; *)

EndPackage[];

