(* -----------------------------------------------------------------

  HTW Prime Knots.m

  Data for HTW prime knots obtained via KnotTheoryByComputer/K2K and Knotscape 
  is stored in subdirectories of "$LGEDirectory/data/Prime Knots".

  ------------------------------------------------------------------ *)

BeginPackage["LinksGouldExplorer`"];

MinimumCrossingNumberOfHTWPrimeKnots::usage = "
  MinimumCrossingNumberOfHTWPrimeKnots.
";

MaximumCrossingNumberOfHTWPrimeKnots::usage = "
  MaximumCrossingNumberOfHTWPrimeKnots.
";

HTWPrimeKnotCrossingNumberLabelSet::usage = "
  HTWPrimeKnotCrossingNumberLabelSet;
";

SoberHTWPrimeKnotCrossingNumberQ::usage = "
  SoberHTWPrimeKnotCrossingNumberQ[c] is True if c is a member of
  HTWPrimeKnotCrossingNumberLabelSet.
";

HTWPrimeKnotAlternatingnessLabelSet::usage = "
  HTWPrimeKnotAlternatingnessLabelSet.
";

SoberHTWPrimeKnotAlternatingnessQ::usage = "
  SoberHTWPrimeKnotAlternatingnessQ[alt] is True if alt is a member of
  HTWPrimeKnotAlternatingnessLabelSet.
";

SoberHTWPrimeKnotSetQ::usage = "
  SoberHTWPrimeKnotSetQ[c, alt] is True if HTWPrimeKnotSet[c, alt] (possibly 
  empty) exists.
";

SoberHTWPrimeKnotQ::usage = "
  SoberHTWPrimeKnotQ[PrimeKnot[c, alt, i]] is True if there is such an HTW 
  prime knot.
";

NonEmptyHTWPrimeKnotCrossingNumberAndAlternatingnessLabelSet::usage = "
  NonEmptyHTWPrimeKnotCrossingNumberAndAlternatingnessLabelSet is the set of 
  labels {c, alt}; such that the sets HTWPrimeKnots[c, alt] each contain some 
  nonzero number of prime knots.
";

NonEmptyHTWPrimeKnotSetQ::usage = "
  NonEmptyHTWPrimeKnotSetQ[c, alt] is True if HTWPrimeKnots[c, alt] exists and is
  nonempty.
";

PrimeKnot::usage = "
  PrimeKnot[c, alt, i] is the HTW prime knot c^alt_i, of sequence number i within
  HTWPrimeKnots[c, alt].
";

HTWPrimeKnots::usage = "
  HTWPrimeKnots[c, alt] is a list of all prime knots of c crossings and 
  alternatingness alt (where the string \"A\" denotes alternating, and \"N\" 
  denotes nonalternating).
";

HTWPrimeKnotsOfUpTocCrossings::usage = "
  HTWPrimeKnotsOfUpTocCrossings[c].
";

NumberOfHTWPrimeKnots::usage = "
  NumberOfHTWPrimeKnots[c, alt] is the number of knots in 
  HTWPrimeKnots[c, alt].
";

NumberOfHTWPrimeKnotsOfUpTocCrossings::usage = "
  NumberOfHTWPrimeKnotsOfUpTocCrossings[c] is the number of knots in
  HTWPrimeKnotsOfUpTocCrossings[c].
";

HTWPrimeKnotsOnsStringBraids::usage = "
  HTWPrimeKnotsOnsStringBraids[c, alt, s] is the subset of
  HTWPrimeKnots[c, alt] of knots whose available (K2K'-) braids are of s
  strings (not of braid index s).
";

HTWPrimeKnotsOfUpTocCrossingsOnsStringBraids::usage = "
  HTWPrimeKnotsOfUpTocCrossingsOnsStringBraids[c, s] is the subset of 
  HTWPrimeKnotsOfUpTocCrossings[c] of knots whose available (K2K'-) braids are 
  of s strings (not of braid index s).
";

NumberOfHTWPrimeKnotsOnsStringBraids::usage = "
  NumberOfHTWPrimeKnotsOnsStringBraids[c, alt, s].
";

NumberOfHTWPrimeKnotsOfUpTocCrossingsOnsStringBraids::usage = "
  NumberOfHTWPrimeKnotsOfUpTocCrossingsOnsStringBraids[c, s].
";

HTWPrimeKnotCrossingNumberAndAlternatingnessLabelSetOfAListOfHTWPrimeKnots::usage = "
  HTWPrimeKnotCrossingNumberAndAlternatingnessLabelSetOfAListOfHTWPrimeKnots[knotlist] 
  is the subset of HTWPrimeKnotCrossingNumberAndAlternatingnessLabelSet found 
  within the given list of HTW prime knots.
";

(* ------------------------------------------------------------------ *)

(* Begin["`Private`"]; *)

MinimumCrossingNumberOfHTWPrimeKnots = 0;

MaximumCrossingNumberOfHTWPrimeKnots = 16;

HTWPrimeKnotCrossingNumberLabelSet =
  Range[MinimumCrossingNumberOfHTWPrimeKnots, MaximumCrossingNumberOfHTWPrimeKnots];

HTWPrimeKnotAlternatingnessLabelSet = {"A", "N"};

SoberHTWPrimeKnotCrossingNumberQ[c_] :=
  MemberQ[HTWPrimeKnotCrossingNumberLabelSet, c];

SoberHTWPrimeKnotAlternatingnessQ[alt_] :=
  MemberQ[HTWPrimeKnotAlternatingnessLabelSet, alt];

HTWPrimeKnotCrossingNumberAndAlternatingnessLabelSet =
  Flatten[
    Outer[List, HTWPrimeKnotCrossingNumberLabelSet, HTWPrimeKnotAlternatingnessLabelSet], 1
  ];

SoberHTWPrimeKnotSetQ[c_, alt_] :=
  MemberQ[HTWPrimeKnotCrossingNumberAndAlternatingnessLabelSet, {c, alt}];

SoberHTWPrimeKnotSetQ[c_] :=
  Apply[Or, Map[SoberHTWPrimeKnotSetQ[c, #]&, HTWPrimeKnotAlternatingnessLabelSet]];

NumberOfHTWPrimeKnots[c_, alt_] :=
  {
    (*  0-crossing *) {           1,            0},
    (*  1-crossing *) {           0,            0},
    (*  2-crossing *) {           0,            0},
    (*  3-crossing *) {           1,            0},
    (*  4-crossing *) {           1,            0},
    (*  5-crossing *) {           2,            0},
    (*  6-crossing *) {           3,            0},
    (*  7-crossing *) {           7,            0},
    (*  8-crossing *) {          18,            3},
    (*  9-crossing *) {          41,            8},
    (* 10-crossing *) {         123,           42},
    (* 11-crossing *) {         367,          185},
    (* 12-crossing *) {        1288,          888},
    (* 13-crossing *) {        4878,         5110},
    (* 14-crossing *) {       19536,        27436},
    (* 15-crossing *) {       85263,       168030},
    (* 16-crossing *) {      379799,      1008906}
(* for good measure, the next few ...
                   17 {     1769979,    "Unknown"},
                   18 {     8400285,    "Unknown"},
                   19 {    40619385,    "Unknown"},
                   20 {   199631989,    "Unknown"},
                   21 {   990623857,    "Unknown"},
                   22 {  4976016485,    "Unknown"},
                   23 { 25182878921,    "Unknown"}
*)
  }[[c+1, Which[alt==="A", 1, alt==="N", 2]]] /;
  SoberHTWPrimeKnotSetQ[c, alt];

NumberOfHTWPrimeKnots[c_] :=
  Apply[Plus, Thread[NumberOfHTWPrimeKnots[c, HTWPrimeKnotAlternatingnessLabelSet]]] /;
    SoberHTWPrimeKnotSetQ[c];

NumberOfHTWPrimeKnotsOfUpTocCrossings[c_] :=
  Apply[Plus, Map[NumberOfHTWPrimeKnots, Range[0, c]]] /;
    SoberHTWPrimeKnotSetQ[c];

HTWPrimeKnots[c_, alt_] :=
  Thread[PrimeKnot[c, alt, Range[NumberOfHTWPrimeKnots[c, alt]]]] /;
    SoberHTWPrimeKnotSetQ[c, alt];

HTWPrimeKnots[c_] :=
  Flatten[Thread[HTWPrimeKnots[c, HTWPrimeKnotAlternatingnessLabelSet]]] /;
    SoberHTWPrimeKnotSetQ[c];

HTWPrimeKnotsOfUpTocCrossings[c_] :=
  Flatten[Map[HTWPrimeKnots, Range[0, c]]] /;
    SoberHTWPrimeKnotSetQ[c];

SoberHTWPrimeKnotQ[PrimeKnot[c_, alt_, i_Integer?Positive]] :=
  (SoberHTWPrimeKnotSetQ[c, alt] && (i <= NumberOfHTWPrimeKnots[c, alt]));

SoberHTWPrimeKnotQ[L_] :=
  False /; (Head[L]=!=PrimeKnot);

HTWPrimeKnotCrossingNumberAndAlternatingnessLabelSetOfAListOfHTWPrimeKnots[knotlist_List] :=
  Union[Cases[knotlist, PrimeKnot[c_, alt_, _] :> {c, alt}]] /;
    Apply[And, Map[(Head[#]===PrimeKnot)&, knotlist]];

NonEmptyHTWPrimeKnotCrossingNumberAndAlternatingnessLabelSet =
  Select[
    HTWPrimeKnotCrossingNumberAndAlternatingnessLabelSet,
    (NumberOfHTWPrimeKnots[Apply[Sequence, #]] =!= 0)&
  ];

NonEmptyHTWPrimeKnotSetQ[c_, alt_] :=
  MemberQ[NonEmptyHTWPrimeKnotCrossingNumberAndAlternatingnessLabelSet, {c, alt}];

EmptyHTWPrimeKnotCrossingNumberAndAlternatingnessLabelSet =
  Complement[
    HTWPrimeKnotCrossingNumberAndAlternatingnessLabelSet,
    NonEmptyHTWPrimeKnotCrossingNumberAndAlternatingnessLabelSet
  ];

(* ----------------------------------------------------------------- *)

HTWPrimeKnotsOnsStringBraids[c_, alt_, s_Integer?Positive] :=
  (HTWPrimeKnotsOnsStringBraids[c, alt, s] =
    Select[HTWPrimeKnots[c, alt], (StringIndex[#]==s)&]
  ) /;
  (
    SoberHTWPrimeKnotSetQ[c, alt]
    &&
    (s <= MaximumCrossingNumberOfHTWPrimeKnots)
  );

HTWPrimeKnotsOnsStringBraids[c_, s_Integer?Positive] :=
  (HTWPrimeKnotsOnsStringBraids[c, s] =
    Select[HTWPrimeKnots[c], (StringIndex[#]==s)&]
  ) /;
  (
    SoberHTWPrimeKnotSetQ[c]
    &&
    (s <= MaximumCrossingNumberOfHTWPrimeKnots)
  );

HTWPrimeKnotsOfUpTocCrossingsOnsStringBraids[c_, s_Integer?Positive] :=
  (HTWPrimeKnotsOfUpTocCrossingsOnsStringBraids[c, s] =
    Select[HTWPrimeKnotsOfUpTocCrossings[c], (StringIndex[#]==s)&]
  ) /;
  (
    SoberHTWPrimeKnotSetQ[c]
    &&
    (s <= MaximumCrossingNumberOfHTWPrimeKnots)
  );

NumberOfHTWPrimeKnotsOnsStringBraids[c_, alt_, s_Integer?Positive] :=
  Length[HTWPrimeKnotsOnsStringBraids[c, alt, s]] /;
  (
    SoberHTWPrimeKnotSetQ[c, alt]
    &&
    (s <= MaximumCrossingNumberOfHTWPrimeKnots)
  );

NumberOfHTWPrimeKnotsOnsStringBraids[c_, s_Integer?Positive] :=
  Length[HTWPrimeKnotsOnsStringBraids[c, s]] /;
  (
    SoberHTWPrimeKnotSetQ[c]
    &&
    (s <= MaximumCrossingNumberOfHTWPrimeKnots)
  );

NumberOfHTWPrimeKnotsOfUpTocCrossingsOnsStringBraids[c_, s_Integer?Positive] :=
  Length[HTWPrimeKnotsOfUpTocCrossingsOnsStringBraids[c, s]] /;
  (
    SoberHTWPrimeKnotSetQ[c]
    &&
    (s <= MaximumCrossingNumberOfHTWPrimeKnots)
  );

(* ----------------------------------------------------------------- *)

(* End[]; *)

EndPackage[];
