(* -----------------------------------------------------------------

  State Model Parameters.m

  State model parameters for LG^{m,n}, in terms of sparse tensors.

  This file needs rewriting. Its functions are out-of-sync with the rest of 
  the code. Some of the material in Radford.smp should be integrated.

  ------------------------------------------------------------------ *)

BeginPackage["LinksGouldExplorer`"];

ScalingFactorsForBraidGeneratorAndHandle::usage = "
  ScalingFactorsForBraidGeneratorAndHandle[Algebra, RepName] returns the 
  scaling factors kappa_S and kappa_C required to convert the R-matrix and 
  `proto-Handle' into a BraidGenerator and Handle, respectively. The \
resulting 
  kappa_S and kappa_C are presented in terms of the internal variables. It 
  would be nicer to be able to partition this code.
";

RepresentationOfTheHandleAsASparseArray::usage = "
  RepresentationOfTheHandleAsASparseArray[Algebra, RepName, s] is the 
  representation of the handle of orientation s (that is +1 or -1) for the 
  Algebra representation RepName.  The automatic construction of S=pi(q^{2 
  h_rho}) for U_q[gl(m|n)] facilitates the discovery of the (positive left) 
  handle C, which is a scaled version of S.
";

RepresentationOfTheHandleAsARank2Array::usage = "
  RepresentationOfTheHandleAsARank2Array[Algebra, RepName, sign].
";

RepresentationOfTheBraidGeneratorAsASparseArray::usage = "
  RepresentationOfTheBraidGeneratorAsASparseArray[Algebra, RepName, N] is the \
\

  representation of the Nth power of the braid generator W^N, for the 
  representation RepName of Algebra. If |N|=1, the representation is a scaled \
\

  version of the appropriate quantum R-matrix. If N=0, the representation is 
  an identity tensor. If |N|>1, the routine uses a recursive bisection 
  strategy.
";

RepresentationOfTheBraidGeneratorAsARank4Array::usage = "
  RepresentationOfTheBraidGeneratorAsARank4Array[Algebra, RepName, N].
";

ConvertFromUnitaryBasisToNonunitaryBasisBraidGenerator::usage = "
  ConvertFromUnitaryBasisToNonunitaryBasisBraidGenerator[Algebra, RepName, \
W].
";

BraidGeneratorAndHandleFirstReidemeisterMoveTest::usage = "
  BraidGeneratorAndHandleFirstReidemeisterMoveTest[Algebra, RepName] is True 
  if the appropriate (representations of the) braid generator and handle 
  together are invariant under the removal (or insertion) of isolated 
  (positive or negative) curls. This means that the associated invariant will \
\

  be invariant under the first Reidemeister move.
";

(* ------------------------------------------------------------------ *)

(* Begin["`Private`"]; *)

ScalingFactorsForBraidGeneratorAndHandle[Algebra_, RepName_] :=
  ScalingFactorsForBraidGeneratorAndHandle[Algebra, RepName] =
  Module[
    {qrmx, R, iqrmx, iR, S, RS, iRS, kappaX, kappaC},

    qrmx = QuantumRMatrix[Algebra, RepName, +1];
    R = StripGradingFromSparseRank4Array[Algebra, RepName, qrmx];
    R = ConvertFromRepresentationToInternalVariables[Algebra, RepName, R];

    iqrmx = QuantumRMatrix[Algebra, RepName, -1];
    iR = StripGradingFromSparseRank4Array[Algebra, RepName, iqrmx];
    iR = ConvertFromRepresentationToInternalVariables[Algebra, RepName, iR];

    S = RepresentationOfqTo2hrho[Algebra, RepName];
    S = ConvertRank2ArrayToRank2SparseTensor[S];
    S = StripGradingFromSparseRank2Array[Algebra, RepName, S];
    S = ConvertFromRepresentationToInternalVariables[Algebra, RepName, S];

    RS = ContractSparseTensorZOverSparseTensorHOnStrings[R, S, 1][[1,2]];
    iRS = ContractSparseTensorZOverSparseTensorHOnStrings[iR, S, 1][[1,2]];

    kappaX = PowerExpand[(iRS/RS)^(1/2)];
    kappaC = PowerExpand[(RS*iRS)^(-1/2)];

    {kappaX, kappaC}
  ];

(* ----------------------------------------------------------------- *)

RepresentationOfTheBraidGeneratorAsASparseArray[Algebra_, RepName_, +1] :=
  (RepresentationOfTheBraidGeneratorAsASparseArray[Algebra, RepName, 1] =
  Module[
    {qrmx, kappaX, R, bg},

   qrmx = QuantumRMatrix[Algebra, RepName, +1];
    R = StripGradingFromSparseRank4Array[Algebra, RepName, qrmx];
    R = ConvertFromRepresentationToInternalVariables[Algebra, RepName, R];
    kappaX = ScalingFactorsForBraidGeneratorAndHandle[Algebra, RepName][[1]];
    R[[All,2]] = R[[All,2]] * kappaX;
    bg = R;

    bg
  ]) /; (Head[Algebra]==Uqgl);

(* ----------------------------------------------------------------- *)

RepresentationOfTheBraidGeneratorAsASparseArray[Algebra_, RepName_, -1] :=
  (RepresentationOfTheBraidGeneratorAsASparseArray[Algebra, RepName, -1] =
  Module[
    {iqrmx, iR, kappaX, ibg},

    iqrmx = QuantumRMatrix[Algebra, RepName, -1];
    iR = StripGradingFromSparseRank4Array[Algebra, RepName, iqrmx];
    iR = ConvertFromRepresentationToInternalVariables[Algebra, RepName, iR];
    kappaX = ScalingFactorsForBraidGeneratorAndHandle[Algebra, RepName][[1]];
    iR[[All,2]] = iR[[All,2]] / kappaX;
    ibg = iR;

    ibg
  ]) /; (Head[Algebra]==Uqgl);

(* Alternative: This is correct, but embeds -signs within square roots, so \
this
  is harder to see. *)

AltRepresentationOfTheBraidGeneratorAsASparseArray[Algebra_, RepName_, -1] :=
  (AltRepresentationOfTheBraidGeneratorAsASparseArray[Algebra, RepName, -1] =
  Module[
    {gr, bg, ibg},

   
    gr = Map[GradingOfBasisVectorOfModule[Algebra, RepName, \
{{e[{},{#}],1}}]&, Range[DimensionOfModule[Algebra, RepName]]];
    bg = RepresentationOfTheBraidGeneratorAsASparseArray[Algebra, RepName, \
+1];
    ibg = bg /. q -> 1/q /. p -> 1/p /. {e[{i_,j_},{k_,l_}],c_} :> \
{e[{j,i},{l,k}],(-1)^((gr[[i]]+gr[[k]])*(gr[[i]]+gr[[l]]))*c};
    ibg = PowerExpand[ibg];
    ibg = TidySparseTensorComponents[ibg];
    ibg = SymmetriseqExpressions[ibg];

    ibg
  ]) /; (Head[Algebra]==Uqgl);

(* ----------------------------------------------------------------- *)

RepresentationOfTheBraidGeneratorAsASparseArray[Algebra_, RepName_, 0] :=
  RepresentationOfTheBraidGeneratorAsASparseArray[Algebra, RepName, 0] =
    IdentitySparseTensor[DimensionOfModule[Algebra, RepName], 2];

RepresentationOfTheBraidGeneratorAsASparseArray[Algebra_, RepName_, \
N_Integer] :=
  (RepresentationOfTheBraidGeneratorAsASparseArray[Algebra, RepName, N] =
  Module[
    {halfN, bg1, bg2, bg},

    halfN = Floor[N/2];
    bg1 = RepresentationOfTheBraidGeneratorAsASparseArray[Algebra, RepName, \
\
halfN];
    bg2 = RepresentationOfTheBraidGeneratorAsASparseArray[Algebra, RepName, \
\
N-halfN];
    bg = AccreteSparseTensorXToSparseTensorZatStrings[bg1, bg2, 1];
    bg[[All, 2]] = Map[Expand, bg[[All, 2]]];

    bg
  ]) /; (Abs[N] > 1);


RepresentationOfTheBraidGeneratorAsARank4Array[Algebra_, RepName_, N_Integer] \
\
:=
  ConvertRank4SparseTensorToRank4Array[
    RepresentationOfTheBraidGeneratorAsASparseArray[Algebra, RepName, N],
    DimensionOfModule[Algebra, RepName]
  ];

(* ----------------------------------------------------------------- *)

RepresentationOfTheHandleAsASparseArray[Algebra_Uqgl, RepName_, 1] :=
  RepresentationOfTheHandleAsASparseArray[Algebra, RepName, 1] =
  Module[
    {S, kappaC, C},

    S = RepresentationOfqTo2hrho[Algebra, RepName];
    S = ConvertRank2ArrayToRank2SparseTensor[S];
    S = StripGradingFromSparseRank2Array[Algebra, RepName, S];
    S = ConvertFromRepresentationToInternalVariables[Algebra, RepName, S];
    C = S;
    kappaC = ScalingFactorsForBraidGeneratorAndHandle[Algebra, RepName][[2]];
    C[[All, 2]] = C[[All, 2]] * kappaC;

    C
  ];

InvertVariables::usage = "
  InvertVariables[Algebra, RepName, S] transforms an expression S viewed as 
  part of the representation theory of the Algebra representation of highest 
  weight RepName, by inverting its variables. The code is 
  representation-dependent.
";

InvertVariables[Uqgl[m_, n_], RepName_, S_] :=
  Module[
    {invrules, iS},

    invrules =
      If[
        (({m,n}=={2,1}) && UseEigenvalueVariablesForLG21Flag),
        {q -> 1/q, p -> 1/p, T0 -> 1/T0, T1 -> 1/T1},
        {q -> 1/q, p -> 1/p, Q -> 1/Q}
      ];
    iS = S /. invrules;

    iS
  ] /; (RepName == LGIHighestWeight[Uqgl[m,n]]);

RepresentationOfTheHandleAsASparseArray[Algebra_Uqgl, RepName_, -1] :=
  RepresentationOfTheHandleAsASparseArray[Algebra, RepName, -1] =
    InvertVariables[
      Algebra, RepName, RepresentationOfTheHandleAsASparseArray[Algebra, \
RepName, 1]
    ];

RepresentationOfTheHandleAsASparseArray[Algebra_, RepName_, -1] :=
  RepresentationOfTheHandleAsASparseArray[Algebra, RepName, +1] /.
    {e[{i_Integer}, {i_}], W_} :> {e[{i}, {i}], 1/W};

RepresentationOfTheHandleAsARank2Array[Algebra_, RepName_, sign_Integer] :=
  ConvertRank2SparseTensorToRank2Array[
    RepresentationOfTheHandleAsASparseArray[Algebra, RepName, sign],
    DimensionOfModule[Algebra, RepName]
  ] /; (Abs[sign] == 1);

(* ----------------------------------------------------------------- *)

ExtraConversionFactors::usage = "
  ExtraConversionFactors[Algebra, RepName] is a utility function for
  ConvertFromUnitaryBasisToNonunitaryBasisBraidGenerator.
";

ExtraConversionFactors[Uqgl[1, 1], {0, alpha}] = {1, (q - 1/q)};

ExtraConversionFactors[Uqgl[2, 1], {0, 0, alpha}] = {1, (q - 1/q), (q - 1/q), \
\
(q - 1/q)^2};

ExtraConversionFactors[Uqgl[3, 1], {0, 0, 0, alpha}] =
  {1, (q - 1/q), (q - 1/q), (q - 1/q), (q - 1/q)^2, (q - 1/q)^2, (q - 1/q)^2, \
\
(q - 1/q)^3};

ConvertFromUnitaryBasisToNonunitaryBasisBraidGenerator[Algebra_, RepName_, \
W_] :=
  Module[
    {mm, tt, sc, f, Xnb},

    mm = MetricMatrixofBPrime[Algebra, RepName];
    tt = ConvertFromRepresentationToInternalVariables[Algebra, RepName, \
Tr[mm, List]];
    sc = ExtraFactors[Algebra, RepName];
    f = (tt * sc)^(-1/2);
    Xnb = W /.
      Simplify[{e[{i_,j_},{k_,l_}], c_} :> {e[{i,j},{k,l}], \
f[[k]]*f[[l]]/(f[[i]]*f[[j]]) * c}];
    Xnb = Expand[PowerExpand[Xnb]];

    Xnb
  ];

(* ----------------------------------------------------------------- *)

BraidGeneratorAndHandleFirstReidemeisterMoveTest[Algebra_, RepName_] :=
  BraidGeneratorAndHandleFirstReidemeisterMoveTest[Algebra, RepName] =
  Module[
    {D, id, bp, bm, hp, hm, hpbp, hpbm, bphm, bmhm, ans},

    D = DimensionOfModule[Algebra, RepName];
    id = IdentityMatrix[D];

    bp = RepresentationOfTheBraidGeneratorAsARank4Array[Algebra, RepName, \
+1];
    bp = ConvertRank4ArrayObjectToRank2ArrayObject[Transpose[bp, {1, 3, 2, \
4}]];
    bm = RepresentationOfTheBraidGeneratorAsARank4Array[Algebra, RepName, \
-1];
    bm = ConvertRank4ArrayObjectToRank2ArrayObject[Transpose[bm, {1, 3, 2, \
4}]];

    hp = Flatten[RepresentationOfTheHandleAsARank2Array[Algebra, RepName, \
+1]];
    hm = Flatten[RepresentationOfTheHandleAsARank2Array[Algebra, RepName, \
-1]];
    hpbp = Partition[Expand[Dot[hp, bp]], D];
    hpbm = Partition[Expand[Dot[hp, bm]], D];
    bphm = Partition[Expand[Dot[bp, hm]], D];
    bmhm = Partition[Expand[Dot[bm, hm]], D];
    ans = (hpbp === hpbm === bphm  === bmhm === id);

    ans
  ];

(* ----------------------------------------------------------------- *)

(* End[]; *)

EndPackage[];


