(* -----------------------------------------------------------------

  Tensor Product Representations.m

  Construct bases for and projectors onto submodules of the tensor product 
  representation of the underlying U_q[gl(m|n)] representation \
pi_{(0|alpha)}.

  This entire file currently being rewritten. Good to "UNDER CONSTRUCTION".

  ------------------------------------------------------------------ *)

BeginPackage["LinksGouldExplorer`"];

WeightsOfModulesinTPDecomposition::usage = "
  WeightsOfModulesinTPDecomposition[Uqgl[m,n], RepName] is the list of the 
  highest weights of the submodules of the orthogonal decomposition of the 
  tensor product of the U_q[gl(m|n)] representation RepName.  The result is 
  based on cite{DeliusGouldLinksZhang:95}, and is only valid for the tensor 
  product of the LGIHighestWeight = (0|alpha) representation with itself.
";

WeightOfTPBasisVector::usage = "
  WeightOfTPBasisVector[Algebra, RepName, {{e[ui, li], 1}}] is the weight of 
  the tensor product basis vector in question.
";


NormOfTPVector::usage = "
  NormOfTPVector[Algebra, RepName, v] is the norm of the tensor product \
vector 
  v.  The dual of |i>o|j> is (-)^([i][j]) <i|o<j|, and the multiplication is 
  graded, that is: (<i|o<j|)(|k>o|l>)=(-)^([j][k]) <i|k>.<j|l>. Together, the \

  norm of |i>o|j> is just <i|i><j|j>, which is 1 if |i> and |j> are \
normalised.
";

NormalisedTPVector::usage = "
  NormalisedTPVector[Algebra, RepName, v] is a normalised version of the 
  tensor product vector v.
";


GradedLeftActionOfAlgebraGeneratorWOnTPVectorv::usage = "
  GradedLeftActionOfAlgebraGeneratorWOnTPVectorv[Algebra, RepName, G, v] is 
  the result of applying the Algebra generator G to the vector expression v.
";

HighestWeightVectorOfTPSubmoduleOfHighestWeightgamma::usage = "
  HighestWeightVectorOfTPSubmoduleOfHighestWeightgamma[Algebra, RepName, 
  gamma] is a highest weight vector for the submodule of highest weight gamma \

  occurring in the orthogonal decomposition of the tensor product of the 
  U_q[gl(m|n)] representation RepName with itself. We suppress the warning 
  message which appears when the \"Solve\" function finds more than one 
  solution.
";

SortTPVectorsIntoDecreasingWeightOrder::usage = "
  SortTPVectorsIntoDecreasingWeightOrder[Algebra, RepName, S] sorts a list of \

  tensor product vectors S into nonincreasing weight order.
";

PartitionTPVectorsIntoWeightClasses::usage = "
  PartitionTPVectorsIntoWeightClasses[Algebra, RepName, S] partitions a 
  (sorted) list of tensor product vectors into weight equivalence classes.
";

ActionOfSeveralGeneratorsGGOnSeveralTPVectorsS::usage = "
  ActionOfSeveralGeneratorsGGOnSeveralTPVectorsS[Algebra, RepName, GG, S] 
  builds a set of tensor product vectors from another such set S under the 
  action of a set of lowering generators GG. The result contains no zeros, is \

  normalised, and is ordered by decreasing weight. It may be {}.
";

BasisForEvenSubAlgebraTPSubmoduleOfHighestWeightgamma::usage = "
  BasisForEvenSubAlgebraTPSubmoduleOfHighestWeightgamma[Algebra, RepName, 
  gamma] is a basis for the even subalgebra submodule of the highest weight 
  module gamma occurring in the orthogonal decomposition of the tensor \
product 
  of the Algebra representation RepName with itself.
";

ConjugateTPVector::usage = "
  ConjugateTPVector[Algebra, RepName, v] is the conjugate of the tensor 
  product vector v.
";

InnerProductofTPVectors::usage = "
  InnerProductofTPVectors[Algebra, RepName, S, T] forms the inner product of 
  the tensor product vectors S and T.
";

OrthonormaliseTPVectorsUsingGramSchmidt::usage = "
  OrthonormaliseTPVectorsUsingGramSchmidt[Algebra, RepName, S] applies a 
  Gram--Schmidt orthonormalisation process to the set of tensor product basis \

  vectors S.
";

BasisForTPModuleOfHighestWeightgamma::usage = "
  BasisForTPModuleOfHighestWeightgamma[Algebra, RepName, gamma] is the tensor \

  product basis for the submodule (of the orthogonal decomposition of the the \

  tensor product of the highest-weight Algebra representation RepName with 
  itself) of highest weight gamma.  It builds the basis by the repeated \
action 
  of all the simple lowering generators on the basis for the even subalgebra 
  module, maintaining the q-graded symmetry of the vectors as it goes.  It 
  orthonormalises the basis and sorts it into decreasing weights.
";

DualBasisForTPModuleOfHighestWeightgamma::usage = "
  DualBasisForTPModuleOfHighestWeightgamma[Algebra, RepName, gamma] is the 
  dual tensor product basis for the submodule (of the orthogonal \
decomposition 
  of the tensor product of the highest-weight Algebra representation RepName 
  with itself) of highest weight gamma, built from 
  BasisForTPModuleOfHighestWeightgamma[Algebra, RepName, gamma].
";

CheckNormalisationOfBasis::usage = "
  CheckNormalisationOfBasis[Algebra, RepName, gamma] checks that the claimed 
  dual bases for the submodule (of the orthogonal decomposition of the tensor \

  product of the highest-weight Algebra representation RepName with itself) \
of 
  highest weight gamma are really so, by reporting their crossover matrix, 
  which should be an identity.
";

ProjectorOntoTPSubmodule::usage = "
  ProjectorOntoTPSubmodule[Algebra, RepName, gamma] is the projector onto the \

  submodule (of the orthogonal decomposition of the tensor product of the 
  highest-weight Algebra representation RepName with itself) of highest \
weight 
  gamma.  The result is returned as a list of nonzero components in terms of 
  the underlying representation variables, although the internals of the 
  construction use the internal variables used in the evaluation of the link 
  invariant.
";

StripGradingFromSparseRank4Array::usage = "
  StripGradingFromSparseRank4Array[Algebra, RepName, S] strips the vector 
  space grading from a sparse rank (2,2) object S.
";

(* ------------------------------------------------------------------ *)

(* Begin["`Private`"]; *)

Needs["DiscreteMath`Combinatorica`"];

AllowableYoungDiagramSet::usage = "
  AllowableYoungDiagramSet[Uqgl[m, n], RepName] is the set of allowable Young \

  diagrams for the U_q[gl(m|n)] representation RepName. It requires 
  \"DiscreteMath`Combinatorica`\" to access \"Partitions\".
";

AllowableYoungDiagramSet[Uqgl[m_, n_], RepName_] :=
  (AllowableYoungDiagramSet[Uqgl[m, n], RepName] =
  Module[
    {yds, ayds},

    yds = Flatten[Map[Partitions, Range[0, m * n]], 1];
    ayds = Select[yds, ((Max[#] <= n) && (Length[#] <= m)) &];

    ayds
  ]) /; (SoberAlgebraWeightQ[Uqgl[m, n], RepName] && (RepName == \
LGIHighestWeight[Uqgl[m,n]]) && (m>=n));

(* ----------------------------------------------------------------- *)

WeightFromAllowableYoungDiagram::usage = "
  WeightFromAllowableYoungDiagram[gl[m, n], ayd] is the gl(m|n) weight 
  associated with an allowable Young diagram ayd. It is a utility function 
  required by WeightsOfModulesinTPDecomposition.
";

WeightFromAllowableYoungDiagram[gl[m_, n_], ayd_List] :=
  (WeightFromAllowableYoungDiagram[gl[m, n], ayd] =
  Module[
    {len, rev, counts, ewt, owt, wt},

    len = Length[ayd];  rev = Reverse[ayd];
    ewt = Join[Table[0, {m - len}], -rev];
    counts = Append[rev, n] - Prepend[rev, 0];
    owt = Flatten[MapThread[Table[#1, {#2}]&, {Range[len,0,-1], counts}]];
    wt = Join[ewt, owt];

    wt
  ]) /; (SoberAlgebraQ[gl[m, n]] && (Length[ayd] <= m) && (Max[ayd] <= n));

(* ----------------------------------------------------------------- *)

WeightsOfModulesinTPDecomposition[Uqgl[m_, n_], RepName_] :=
  (WeightsOfModulesinTPDecomposition[Uqgl[m, n], RepName] =
  Module[
    {ayds, wts},

    ayds = AllowableYoungDiagramSet[Uqgl[m, n], RepName];
    wts = Map[2 RepName + WeightFromAllowableYoungDiagram[gl[m, n], #]&, \
ayds];

    wts
  ]) /; (SoberAlgebraWeightQ[Uqgl[m, n], RepName] && (RepName == \
LGIHighestWeight[Uqgl[m,n]]) && (m>=n));

(* ----------------------------------------------------------------- *)

(* UNDER CONSTRUCTION -- tidy this function *)

(*
  WeightOfTPBasisVector should work for TPs of all orders, even 1.
  It depends on the rewritten WeightOfBasisVectorOfModule
*)

WeightOfBasisVectorOfModule[Algebra_, RepName_, \
{{e[{},{i_Integer?Positive}],1}}] :=
  (WeightOfBasisVectorOfModule[Algebra, RepName, {{e[{},{i}],1}}] =
    WeightOfBasisVectorOfModule[Algebra, RepName, ket[i]]
  ) /; SoberAlgebraWeightQ[Algebra, RepName];

WeightOfTPBasisVector[Algebra_, RepName_, {{e[{}, li_List?VectorQ], 1}}] :=
  (WeightOfTPBasisVector[Algebra, RepName, {{e[{}, li], 1}}] =
    Apply[Plus, Map[WeightOfBasisVectorOfModule[Algebra, RepName, \
{{e[{},{#}], 1}}]&, li]]
  ) /; SoberAlgebraWeightQ[Algebra, RepName] && (Length[li] > 0);

(*
  This is for the underlying module! New notation ...
*)

GradingOfBasisVectorOfModule[Algebra_, RepName_, {{e[{}, \
{i_Integer?Positive}], 1}}] :=
  GradingOfBasisVectorOfModule[Algebra, RepName, ket[i]] /;
    SoberBasisVectorOfModule[Algebra, RepName, ket[i]];

(* ----------------------------------------------------------------- *)

NormOfTPVector[Algebra_, RepName_, v_List] :=
  CleanqExpressions[Sqrt[Apply[Plus, v[[All, 2]]^2]]] /;
    SoberAlgebraWeightQ[Algebra, RepName];

NormalisedTPVector[Algebra_, RepName_, {{e[ui_List?VectorQ, li_List?VectorQ], \
0}}] :=
  {{e[ui, li], 0}} /;
    SoberAlgebraWeightQ[Algebra, RepName];

NormalisedTPVector[Algebra_, RepName_, v_] :=
  Module[
    {vout, nv},

    nv = NormOfTPVector[Algebra, RepName, v];
    vout = v;
    vout[[All, 2]] = Map[CleanqExpressions, vout[[All, 2]]/nv];

    vout
  ] /; SoberAlgebraWeightQ[Algebra, RepName];

(* ----------------------------------------------------------------- *)

(* Only works for twofold TP. Clumsy but seems functional. *)

GradedLeftActionOfAlgebraGeneratorWOnTPVectorv[Algebra_, RepName_, \
G_Generator, v_List] :=
  Module[
    {cpG, tv, Gv},

    cpG = Flatten[{CoProduct[G]} /. Plus -> List];
    cpG = cpG /. Times[coeff_:1, TensorProduct[A__]] :> {TensorProduct[A], \
coeff};

    tv = v /. {e[{},{i_, j_}], coeff_} :> \
{TensorProduct[e[{},{i}],e[{},{j}]], coeff};
    Gv = Flatten[Outer[({AWZ[#1[[1]], #2[[1]]], #1[[2]] * #2[[2]]})&, cpG, \
tv, 1], 1];
    Gv = Gv /. {AWZ[TensorProduct[A_, B_], TensorProduct[C_, D_]], coeff_} :>
      TensorProduct[AWZ[A, {{C, 1}}], AWZ[B, {{D, \
(-1)^(GradingOfAlgebraGenerator[Algebra, \
B]*GradingOfBasisVectorOfModule[Algebra, RepName,{{C,1}}]) * coeff}}]];
    Gv = Gv /. NonCommutativeMultiply -> NCM;
    Gv = Gv /. A_Generator :> Representation[Algebra, RepName, A] /. NCM -> \
Dot;
    Gv = Gv /. AWZ[A_, B_] :> AWZ[ConvertRank2ArrayToRank2SparseTensor[A], \
B];
    Gv = Gv /. AWZ[A_, B_] :> AccreteSparseTensorWToSparseTensorZatStrings[B, \
"A", 1];
    Gv = Flatten[Gv /. Kron -> SparseTensorProductOfSparseTensors, 1];
    Gv = TidySparseTensorComponents[Gv];
    Gv = CleanqExpressions[Gv];

    Gv
  ] /; SoberAlgebraWeightQ[Algebra, RepName];

(* ----------------------------------------------------------------- *)

HighestWeightVectorOfTPSubmoduleOfHighestWeightgamma[Algebra_, RepName_, \
gamma_List] :=
  (HighestWeightVectorOfTPSubmoduleOfHighestWeightgamma[Algebra, RepName, \
gamma] =
  Module[
    {D, tpbvlist, vars, hwv, srgs, raisedhwv, coefflist, eqns, subrules},

    D = DimensionOfModule[Algebra, RepName];
    tpbvlist =
      Select[
        Flatten[Outer[{{e[{},{#1,#2}],1}}&, Range[D], Range[D]], 1],
        (WeightOfTPBasisVector[Algebra, RepName, #] == gamma)&
      ];
    vars = Array[theta, Length[tpbvlist]];
    hwv = Transpose[{tpbvlist[[All, 1, 1]], vars}];
    srgs = Select[GeneratorSet[Algebra, {0,1}, {+1}], \
SimpleAlgebraGeneratorQ];
    raisedhwv = Map[GradedLeftActionOfAlgebraGeneratorWOnTPVectorv[Algebra, \
RepName,#,hwv]&, srgs];
    coefflist = Flatten[raisedhwv, 1][[All, 2]];
    eqns = Map[(# == 0)&, coefflist];
    Off[Solve::svars];
    subrules = Flatten[Solve[eqns, vars]];
    hwv = (hwv /. subrules) /. theta[_] :> 1;
    hwv = NormalisedTPVector[Algebra, RepName, hwv];

    hwv
  ]) /;
  (
    SoberAlgebraWeightQ[Algebra, RepName]
    &&
    (RepName == LGIHighestWeight[Algebra])
    &&
    MemberQ[WeightsOfModulesinTPDecomposition[Algebra, RepName], gamma, {1}]
  );

(* ----------------------------------------------------------------- *)

SortTPVectorsIntoDecreasingWeightOrder[Algebra_, RepName_, S_List] :=
  Module[
    {wts, wtswithS, sortedwtswithS, Ssorted},

    wts = Map[WeightOfTPBasisVector[Algebra, RepName,{{#,1}}]&, \
S[[All,1,1]]];
    wtswithS = Transpose[{wts, S}];
    sortedwtswithS = Sort[wtswithS, OrderedWeightsQ[#2[[1]],#1[[1]]]&];
    Ssorted = Transpose[sortedwtswithS][[2]];

    Ssorted
  ] /; SoberAlgebraWeightQ[Algebra, RepName];

(* ----------------------------------------------------------------- *)

PartitionTPVectorsIntoWeightClasses[Algebra_, RepName_, S_List] :=
  Module[
    {wts, wtswithS, SplitwtswithS, SplitS},

    wts = Map[WeightOfTPBasisVector[Algebra, RepName, {{#,1}}]&, \
S[[All,1,1]]];
    wtswithS = Transpose[{wts, S}];
    wtswithS = Sort[wtswithS, OrderedWeightsQ[#2[[1]], #1[[1]]]&];
    SplitwtswithS = Split[wtswithS, (#1[[1]] == #2[[1]])&];
    SplitS = Map[Transpose[#][[2]]&, SplitwtswithS, {1}];

    SplitS
  ] /; SoberAlgebraWeightQ[Algebra, RepName];

(* ----------------------------------------------------------------- *)

ActionOfSeveralGeneratorsGGOnSeveralTPVectorsS[Algebra_, RepName_, {}, \
S_List] :=
  {} /; SoberAlgebraWeightQ[Algebra, RepName];

ActionOfSeveralGeneratorsGGOnSeveralTPVectorsS[Algebra_, RepName_, GG_List, \
{}] :=
  {} /; SoberAlgebraWeightQ[Algebra, RepName];

ActionOfSeveralGeneratorsGGOnSeveralTPVectorsS[Algebra_, RepName_, GG_List, \
S_List] :=
  Module[
    {Snew},

    Snew =
      Flatten[
        Outer[GradedLeftActionOfAlgebraGeneratorWOnTPVectorv[Algebra, \
RepName,#1,#2]&,GG,S,1],
        1
      ];
    Snew = Map[TidySparseTensorComponents, Snew];
    Snew = Map[NormalisedTPVector[Algebra, RepName, #]&, Snew];
    Snew = SortTPVectorsIntoDecreasingWeightOrder[Algebra, RepName,Snew];
    Snew = Complement[Snew, {{{e[{},{1,1}],0}}}];

    Snew
 ] /; SoberAlgebraWeightQ[Algebra, RepName];

(* ----------------------------------------------------------------- *)

BasisForEvenSubAlgebraTPSubmoduleOfHighestWeightgamma[Algebra_, RepName_, \
gamma_List] :=
  (BasisForEvenSubAlgebraTPSubmoduleOfHighestWeightgamma[Algebra, RepName, \
gamma] =
  Module[
    {selgs, Ssub, Sold, Snew, Stemp, Sout},

    selgs = Select[GeneratorSet[Algebra, {0}, {-1}], \
SimpleAlgebraGeneratorQ];
    Ssub = {HighestWeightVectorOfTPSubmoduleOfHighestWeightgamma[Algebra, \
RepName, gamma]};
    Sold = Ssub;
    Snew = Null;

    While[
      (Snew =!= {}),
      Snew = ActionOfSeveralGeneratorsGGOnSeveralTPVectorsS[Algebra, RepName, \
selgs, Sold];
      Snew = Complement[Snew, Ssub];
      Ssub = Join[Ssub, Snew];
      Sold = Snew
    ];
    Sout = SortTPVectorsIntoDecreasingWeightOrder[Algebra, RepName,Ssub];

    Sout
  ]) /;
  (
    SoberAlgebraWeightQ[Algebra, RepName]
    &&
    (RepName == LGIHighestWeight[Algebra])
    &&
    MemberQ[WeightsOfModulesinTPDecomposition[Algebra, RepName], gamma, {1}]
  );

(* ----------------------------------------------------------------- *)

ConjugateTPVector[Algebra_, RepName_, v_List] :=
  Module[
    {gr, vconj},

    gr = Map[GradingOfBasisVectorOfModule[Algebra, RepName, \
{{e[{},{#}],1}}]&, Range[DimensionOfModule[Algebra, RepName]]];
    vconj = v /. {e[{},{i_,j_}], c_} :> {e[{i,j},{}], (-1)^(gr[[i]] * \
gr[[j]]) * c};

    vconj
  ] /; SoberAlgebraWeightQ[Algebra, RepName];

(* ----------------------------------------------------------------- *)

InnerProductofTPVectors[Algebra_, RepName_, v_List, w_List] :=
  Module[
    {ip},

    ip = Apply[Plus, Flatten[
      Outer[KroneckerDelta, v[[All,1,2]], w[[All,1,2]],1] * \
Outer[Times,v[[All,2]],w[[All,2]]]
    ]];
    ip = CleanqExpressions[ip];

    ip
  ] /; SoberAlgebraWeightQ[Algebra, RepName];

(* ----------------------------------------------------------------- *)

OrthonormaliseTPVectorsUsingGramSchmidt[Algebra_, RepName_, S_List] :=
  Map[NormalisedTPVector[Algebra, RepName, #]&, S] /;
  (
    SoberAlgebraWeightQ[Algebra, RepName]
    &&
    (RepName == LGIHighestWeight[Algebra])
    &&
    (Length[S]==1)
  );

OrthonormaliseTPVectorsUsingGramSchmidt[Algebra_, RepName_, S_List] :=
  Module[
    {Sout, LL, ips, j},

    WriteString["stdout", "OrthonormaliseTPVectorsUsingGramSchmidt[", \
Length[S], " vectors]: "];

    Sout = Table[ZeroSparseTensor[{0, 2}], {Length[S]}];
    Do[
      WriteString["stdout", " ", j];
      LL = Take[Sout, j-1];
      ips = Flatten[Map[InnerProductofTPVectors[Algebra, RepName, #, \
S[[j]]]&, LL], 1];
      LL[[All, All, 2]] = - LL[[All, All, 2]] * ips;
      Sout[[j]] = Join[S[[j]], Flatten[LL, 1]];
      Sout[[j]] = TidySparseTensorComponents[Sout[[j]]];
      Sout[[j]] = CleanqExpressions[Sout[[j]]];
      Sout[[j]] = NormalisedTPVector[Algebra, RepName, Sout[[j]]],
      {j, Length[S]}
    ];
    WriteString["stdout", " done.\n"];

    Sout
  ] /;
  (
    SoberAlgebraWeightQ[Algebra, RepName]
    &&
    (RepName == LGIHighestWeight[Algebra])
    &&
    (S=!={})
  );

(* ----------------------------------------------------------------- *)

BasisForTPModuleOfHighestWeightgamma[Algebra_, RepName_, gamma_List] :=
  (BasisForTPModuleOfHighestWeightgamma[Algebra, RepName, gamma] =
  Module[
    {solgs, selgs, i, Ssub, Sold, Snew, Snewsub, Snewold, Snewnew, Ssorted, \
Spartitioned, Sout},

    WriteString["stdout", "BasisForTPModuleOfHighestWeightgamma[", Algebra,", \
", RepName, ", ", gamma, "]\n"];
    solgs = Select[GeneratorSet[Algebra, {1}, {-1}], \
SimpleAlgebraGeneratorQ];
    selgs = Select[GeneratorSet[Algebra, {0}, {-1}], \
SimpleAlgebraGeneratorQ];

    Ssub = BasisForEvenSubAlgebraTPSubmoduleOfHighestWeightgamma[Algebra, \
RepName,gamma];
    Sold = Ssub;
    Snew = Null;

    i = 1;
    While[
      (Snew =!= {}),
      Snewsub = ActionOfSeveralGeneratorsGGOnSeveralTPVectorsS[Algebra, \
RepName, solgs, Sold];
      Snewold = Snewsub;
      Snewnew = Null;
      While[
        (Snewnew =!= {}),
        Snewnew = ActionOfSeveralGeneratorsGGOnSeveralTPVectorsS[Algebra, \
RepName, selgs, Snewold];
        Snewnew = Complement[Snewnew,Snewsub];
        Snewsub = Join[Snewsub, Snewnew];
        Snewold = Snewnew
      ];
      Snew = Snewsub;

      Snew = Complement[Snew, Ssub];
      WriteString[
        "stdout",
        "  Level ", i++, ": Ssub ", Length[Ssub],
        ", Sold ", Length[Sold], ", Snew ", Length[Snew], " vectors.\n"
      ];
      Ssub = Join[Ssub, Snew];
      Sold = Snew
    ];

    Ssorted = SortTPVectorsIntoDecreasingWeightOrder[Algebra, RepName, Ssub];
    Spartitioned = PartitionTPVectorsIntoWeightClasses[Algebra, RepName, \
Ssorted];
    Sout = Map[OrthonormaliseTPVectorsUsingGramSchmidt[Algebra, RepName, #]&, \
Spartitioned];
    Sout = Flatten[Sout,1];

    Sout
  ]) /;
  (
    SoberAlgebraWeightQ[Algebra, RepName]
    &&
    (RepName == LGIHighestWeight[Algebra])
    &&
    MemberQ[WeightsOfModulesinTPDecomposition[Algebra, RepName], gamma, {1}]
  );

(* ----------------------------------------------------------------- *)

DualBasisForTPModuleOfHighestWeightgamma[Algebra_, RepName_, gamma_List] :=
  Map[ConjugateTPVector[Algebra, RepName, #]&, \
BasisForTPModuleOfHighestWeightgamma[Algebra, RepName,gamma]] /;
  (
    SoberAlgebraWeightQ[Algebra, RepName]
    &&
    (RepName == LGIHighestWeight[Algebra])
    &&
    MemberQ[WeightsOfModulesinTPDecomposition[Algebra, RepName], gamma, {1}]
  );

(* ----------------------------------------------------------------- *)

CheckNormalisationOfBasis[Algebra_, RepName_, gamma_List] :=
  Module[
    {bfm, mx},

    bfm = BasisForTPModuleOfHighestWeightgamma[Algebra, RepName, gamma];
    mx = Outer[InnerProductofTPVectors[Algebra, RepName, #1, #2]&, bfm, bfm, \
1];

    mx
  ] /;
  (
    SoberAlgebraWeightQ[Algebra, RepName]
    &&
    (RepName == LGIHighestWeight[Algebra])
    &&
    MemberQ[WeightsOfModulesinTPDecomposition[Algebra, RepName], gamma, {1}]
  );

(* ----------------------------------------------------------------- *)

DyadOfTPVectorsvAndw::usage = "
  DyadOfTPVectorsvAndw[Algebra, RepName, v, w] is the dyad of vectors v and \
w. 
  This is a utility function called by ProjectorOntoTPSubmodule.
";

DyadOfTPVectorsvAndw[Algebra_, RepName_, v_List, w_List] :=
  Module[
    {gr, dy},
    gr = Map[GradingOfBasisVectorOfModule[Algebra, RepName, \
{{e[{},{#}],1}}]&, Range[DimensionOfModule[Algebra, RepName]]];
    dy = Outer[{e[#1[[1,2]], #2[[1,1]]], (-1)^(gr[[#1[[1,2,2]]]] * \
gr[[#2[[1,1,1]]]])*#1[[2]]*#2[[2]]}&, v, w, 1];

    dy
  ] /; SoberAlgebraWeightQ[Algebra, RepName];

(* Previous versions of this used variable p, which may speed things up. *)

ProjectorOntoTPSubmodule[Algebra_, RepName_, gamma_List] :=
  (ProjectorOntoTPSubmodule[Algebra, RepName, gamma] =
  Module[
    {bfm, dbfm, ptp},

    WriteString["stdout", "ProjectorOntoTPSubmodule[", Algebra, ", ", \
RepName, ", ", gamma, "] ... "];
    bfm = BasisForTPModuleOfHighestWeightgamma[Algebra, RepName, gamma];
    dbfm = DualBasisForTPModuleOfHighestWeightgamma[Algebra, RepName, gamma];
    ptp = Flatten[Inner[DyadOfTPVectorsvAndw[Algebra, RepName, #1, #2]&, bfm, \
dbfm, List], 2];
    ptp = TidySparseTensorComponents[ptp];
    ptp = CleanqExpressions[ptp];
    WriteString["stdout", Length[ptp], " components.\n"];

    ptp
  ]) /;
  (
    SoberAlgebraWeightQ[Algebra, RepName]
    &&
    (RepName == LGIHighestWeight[Algebra])
    &&
    MemberQ[WeightsOfModulesinTPDecomposition[Algebra, RepName], gamma, {1}]
  );

(* ----------------------------------------------------------------- *)

StripGradingFromSparseRank4Array[Algebra_, RepName_, S_] :=
  Module[
    {gr, Sstrip},

    gr = GradingListOfBasisVectorsOfModule[Algebra, RepName];
    Sstrip = S /. {e[{i_,j_}, {k_,l_}], W_} :> {e[{i,j}, {k,l}], \
(-1)^(gr[[k]]*(gr[[j]] + gr[[l]])) * W};

    Sstrip
  ] /; SoberAlgebraWeightQ[Algebra, RepName];

(* ----------------------------------------------------------------- *)

(* End[]; *)

EndPackage[];

