(* -----------------------------------------------------------------

  Torus Knots.m

  ------------------------------------------------------------------ *)

BeginPackage["LinksGouldExplorer`"];

TorusKnotQ::usage = "
  TorusKnotQ[L] is True if link L is some TorusKnot[m, n] or a torus HTW prime 
  knot, and False for other HTW prime knots.
";

TorusKnot::usage = "
  TorusKnot[m, n] is the (m,n) torus knot.
";

TorusCode::usage = "
  TorusCode[L] of a torus knot L is {m,n} such that L is TorusKnot[m,n].
";

TorusHTWPrimeKnots::usage = "
  TorusHTWPrimeKnots[c, alt] is the set of torus HTW prime knots within
  HTWPrimeKnots[c, alt].
";

TorusHTWPrimeKnotsOfUpTocCrossings::usage = "
  TorusHTWPrimeKnotsOfUpTocCrossings[c] is the set of torus HTW prime knots 
  within HTWPrimeKnotsOfUpTocCrossings[c].
";

(* ------------------------------------------------------------------ *)

(* Begin["`Private`"]; *)

TorusKnot[m_Integer?Negative, n_Integer?Negative] :=
  TorusKnot[-m, -n];

LinkReflection[TorusKnot[m_Integer, n_Integer]] :=
  TorusKnot[m, -n];

LinkInversion[TorusKnot[m_Integer, n_Integer]] :=
  TorusKnot[m, n];

MapThread[(TorusHTWPrimeKnots[##] = {})&, HTWPrimeKnotCrossingNumberAndAlternatingnessLabelSet];

TorusHTWPrimeKnots[ 0, "A"] = {PrimeKnot[ 0, "A",      1]};
TorusHTWPrimeKnots[ 3, "A"] = {PrimeKnot[ 3, "A",      1]};
TorusHTWPrimeKnots[ 5, "A"] = {PrimeKnot[ 5, "A",      2]};
TorusHTWPrimeKnots[ 7, "A"] = {PrimeKnot[ 7, "A",      7]};
TorusHTWPrimeKnots[ 8, "A"] = {PrimeKnot[ 8, "N",      3]};
TorusHTWPrimeKnots[ 9, "A"] = {PrimeKnot[ 9, "A",     41]};
TorusHTWPrimeKnots[10, "N"] = {PrimeKnot[10, "N",     21]};
TorusHTWPrimeKnots[11, "A"] = {PrimeKnot[11, "A",    367]};
TorusHTWPrimeKnots[13, "A"] = {PrimeKnot[13, "A",   4878]};
TorusHTWPrimeKnots[14, "N"] = {PrimeKnot[14, "N",  21881]};
TorusHTWPrimeKnots[15, "A"] = {PrimeKnot[15, "A",  85263]};
TorusHTWPrimeKnots[15, "N"] = {PrimeKnot[15, "N",  41185]};
TorusHTWPrimeKnots[16, "N"] = {PrimeKnot[16, "N", 783154]};

TorusHTWPrimeKnots[c_] :=
  Flatten[Map[TorusHTWPrimeKnots[c, #]&, {"A", "N"}]] /;
    SoberHTWPrimeKnotSetQ[c];

TorusHTWPrimeKnotsOfUpTocCrossings[c_] :=
  Flatten[Apply[TorusHTWPrimeKnots, Select[HTWPrimeKnotCrossingNumberAndAlternatingnessLabelSet, (#[[1]] <= c)&], 1]] /;
    SoberHTWPrimeKnotSetQ[c];

TorusKnotQ[PrimeKnot[c_, alt_, i_]] :=
  MemberQ[TorusHTWPrimeKnots[c, alt], PrimeKnot[c, alt, i]] /;
    SoberHTWPrimeKnotQ[PrimeKnot[c, alt, i]];

TorusKnotQ[TorusKnot[m_Integer?Positive, n_Integer?Positive]] :=
  True;

TorusCode[PrimeKnot[ 0, "A",      1]] = { 1, 2};
TorusCode[PrimeKnot[ 3, "A",      1]] = { 3, 2};
TorusCode[PrimeKnot[ 5, "A",      2]] = { 5, 2};
TorusCode[PrimeKnot[ 7, "A",      7]] = { 7, 2};
TorusCode[PrimeKnot[ 8, "N",      3]] = { 4, 3};
TorusCode[PrimeKnot[ 9, "A",     41]] = { 9, 2};
TorusCode[PrimeKnot[10, "N",     21]] = { 5, 3};
TorusCode[PrimeKnot[11, "A",    367]] = {11, 2};
TorusCode[PrimeKnot[13, "A",   4878]] = {13, 2};
TorusCode[PrimeKnot[14, "N",  21881]] = { 7, 3};
TorusCode[PrimeKnot[15, "A",  85263]] = {15, 2};
TorusCode[PrimeKnot[15, "N",  41185]] = { 5, 4};
TorusCode[PrimeKnot[16, "N", 783154]] = { 8, 3};

TorusCode[TorusKnot[m_Integer?Positive, n_Integer?Positive]] :=
  {m, n};

(* End[]; *)

EndPackage[];
