(* Mathematica Version:5.0 *)

(* Name : `PDconvert` *)

(* Title:PDconvert *)

(* Authors:  S. Jablan, R. Sazdanovic *)

(* Copyright: Copyright 2006 *)

(* History: current Version 1.1 at 10.02.2004 *)

(* Summary: This package provides functions converting LinKnot data to PD *)

(* Context: `PDconvert` *)

(* Keywords: Knot Theory, Knots, Links, Conway notation, PD *)

(* Requirements: None *)

BeginPackage["PDconvert`"]; 




(*####################### CONVERSIONS TO PD #########################*)


checkArgs[s_,t_]:=
  ListQ[s]&&VectorQ[t,IntegerQ[#]&&#\[GreaterEqual]0&]&&
    Tr[t]\[LessEqual]Length[s]
    
    
iteratedTake[s_,t_]/;checkArgs[s,t]:=
  iteratedTake[s,t]=
    With[{w=FoldList[Plus,0,t]},
      Map[Take[s,#]&,Transpose[{Drop[w,-1]+1,Rest[w]}]]]

fConToPD[Ul_String] := Module[{mm, nn, ss, vv, i}, mm = fGaussExtSigns[Ul];
    nn = fGaussExtSigns[StringReplace[Ul, "-" -> ""]];
    nn = Map[Sign, Flatten[mm]]*Map[Sign, Flatten[nn]];
    vv = Table[nn[[i]]*(-1)^i, {i, Length[nn]}];
    vv = Abs[Flatten[mm]]*vv;
    ss = Map[Length, mm];
    mm = If[MemberQ[ss, 0], {vv}, iteratedTake[vv, ss]];
    mm]


fConwayToPD[Ul_String] := Module[{ss}, ss = fConToPD[Ul];
    ss = PD[GaussCode @@ ss];
    ss]


fKnDowToPD[Ul_List] := Module[{ss, gg, sc, i}, ss = Map[Sign, Ul[[2]]];
    gg = Map[Sort, Table[{2i - 1, Abs[Ul[[2, i]]]}, {i, Length[Ul[[2]]]}]];
    sc = Flatten[
        Complement[
          Table[If[ss[[i]] < 0, gg[[i]], {}], {i, Length[ss]}], {{}}]];
    gg = Map[Last, 
        Sort[Flatten[
            Table[{{gg[[i, 1]], i}, {gg[[i, 2]], i}}, {i, Length[gg]}], 1]]];
    gg = Table[gg[[i]]*(-1)^i, {i, Length[gg]}];
    gg = Table[If[MemberQ[sc, i], -gg[[i]], gg[[i]]], {i, Length[gg]}];
    gg = If[Length[Ul[[1]]] == 1, {gg}, iteratedTake[gg, 2Ul[[1]]]];
    gg]

fKnotscapeDowToPD[Ul_List] := Module[{ss}, ss = fKnDowToPD[Ul];
    ss = PD[GaussCode @@ ss];
    ss]

fDowkerToPD[Ul_List] := Module[{ss, ss1, i}, ss = fSignsKL[Abs[Ul]];
    ss1 = Map[Sign, Ul[[2]]]*Map[Sign, ss[[2]]];
    ss = {Ul[[1]], ss1*Ul[[2]]};
    ss = fKnDowToPD[ss];
    ss = PD[GaussCode @@ ss];
    ss]

fPdataToPD[Ul_List] := Module[{pd}, pd = fDowkerToPD[fDowfromPD[Ul]];
    pd]


EndPackage[] 






