(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    203860,       7809]*)
(*NotebookOutlinePosition[    204565,       7833]*)
(*  CellTagsIndexPosition[    204521,       7829]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[StyleBox["LinKnot (K2KC) \n\tReference Manual",
  FontColor->RGBColor[0.500008, 0, 0]]], "Title",
  CellFrame->True,
  Background->GrayLevel[0.666667]],

Cell[TextData[StyleBox["Slavik Jablan, Radmila Sazdanovic",
  FontColor->RGBColor[0.500008, 0, 0]]], "Subtitle",
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["The Mathematical Institute\nKnez Mihailova 35, P.O. Box 367,\n\
11001 Belgrade, Serbia & Montenegro",
    FontSize->18,
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0]],
  StyleBox["\njablans",
    FontColor->RGBColor[0.500008, 0, 0]],
  StyleBox["@mi.sanu.ac.yu, seasmile@etf.bg.ac.yu",
    FontSize->14,
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0]]
}], "Subsubtitle",
  Background->None],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Functions",
  FontVariations->{"CompatibilityType"->0}]], "Subtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.666667]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["AllStatesRational",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["AllStatesRational",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Conway_String\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["AllStatesRational",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "] calculates  all states of a given rational ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " given by its Conway symbol, i.e.,  all different variations of signs in a \
given projection of a rational  ",
  StyleBox["KL",
    FontSlant->"Italic"],
  ". The first datum in the result is the unknotting (unlinking)  number \
obtained from the fixed projection of a given rational ",
  StyleBox["KL",
    FontSlant->"Italic"],
  ", followed by  the list of minimum numbers of crossing changes necessary  \
to obtain the corresponding rational ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " given by its Conway symbol,  i.e., the list of the ",
  StyleBox["KL ",
    FontSlant->"Italic"],
  "distances from a given rational ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " projection."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["BreakCoAll",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["BreakCoAll",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Ul_",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string,  Dowker code as a list, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  " as a list."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["BreakCoAll",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "]",
  StyleBox[" ",
    FontWeight->"Bold"],
  "in a link given by its Conway  symbol, Dowker code, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  ", cutts all components. The results are all different ",
  StyleBox["KL",
    FontSlant->"Italic"],
  "s obtained by cutting all components, where  for unknot (unlink) the \
result is {0}."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["CuttNo",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["CuttNo",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Ul_",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string,  Dowker code as a list, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  " as a list."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["CuttNo",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "]",
  StyleBox[" ",
    FontWeight->"Bold"],
  "calculates the cutting number of a   link given by its Conway symbol, \
Dowker code, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  "."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Dow",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Dow",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Conway_String\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Dow",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "] produces Dowker code of a knot or link, as a list ",
  StyleBox["Dow",
    FontSlant->"Italic"],
  ". "
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["External Program", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell["None.", "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Notes", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  "* The first data of the list ",
  StyleBox["Dow",
    FontSlant->"Italic"],
  " obtained as a result are lengths of components (the number of crossings \
of each component), and the second Dowker code (number sequence of even \
numbers derived from the knot or link with the signs of crossing points).",
  StyleBox[" ",
    FontVariations->{"CompatibilityType"->0}]
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["DrawPlanarEmbKL",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["DrawPlanarEmbKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Ul_\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string,  Dowker code as a list, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  " as a list."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["External Program", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell["planarity.exe", "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["DrawPlanarEmbKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "] draws the planar embedding of a prime KL given by its Conway symbol, \
Dowker core or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  StyleBox[",  without showing digonal faces. As the basis of this program it \
is used the program \"3-dimensional Convex Drawings of 3-connected Planar \
Graphs\" by M.Ochiai and N.Imafuji.",
    FontVariations->{"CompatibilityType"->0}]
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["DrawPlanarEmbGraph",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["DrawPlanarEmbGraph",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_List",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Ul_List\t\t",
    FontSlant->"Italic"],
  " Graph ",
  StyleBox["G",
    FontSlant->"Italic"],
  " given by a list of unordered pairs of vertices."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["External Program", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell["", "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["DrawPlanarEmbGraph",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "] draws the planar embedding of a graph ",
  StyleBox["G",
    FontSlant->"Italic"],
  " given by a list of unordered pairs of vertices",
  StyleBox[", without showing multiple edges. As the basis of this program it \
is used the program \"3-dimensional Convex Drawings of 3-connected Planar \
Graphs\" by M.Ochiai and N.Imafuji.",
    FontVariations->{"CompatibilityType"->0}]
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["GetKnotLink",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["GetKnotLink",
    FontWeight->"Bold"],
  "[",
  StyleBox["Str",
    FontSlant->"Italic"],
  StyleBox["_String,No_Integer",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Str_String\t",
    FontSlant->"Italic"],
  StyleBox["Str",
    FontSlant->"Italic"],
  " is the name of a list of knots and links with a specified number of \
crossings  (for example,",
  StyleBox[" ",
    FontSlant->"Italic"],
  StyleBox["\"",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["a",
    FontSlant->"Italic"],
  "10\" or \"",
  StyleBox["n",
    FontSlant->"Italic"],
  "10\", where ",
  StyleBox["a",
    FontSlant->"Italic"],
  " stands for alternating and ",
  StyleBox["n ",
    FontSlant->"Italic"],
  "for non-alternating  knots and links, written as a string), and ",
  StyleBox["No",
    FontSlant->"Italic"],
  StyleBox[" is",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" ",
    FontSlant->"Italic"],
  "the number of a desired knot ot link from that list."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["GetKnotLink",
    FontWeight->"Bold",
    FontColor->RGBColor[0.500008, 0, 0]],
  "[",
  StyleBox["Str",
    FontSlant->"Italic"],
  StyleBox["_String,No_Integer",
    FontSlant->"Italic"],
  "]",
  StyleBox[" ",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["returns as the result a Conway symbol of  the desired knot or \
link, that can be used for further calculations.",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}]
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["External Program", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[StyleBox["None.",
  FontVariations->{"CompatibilityType"->0}]], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["LiangPoly",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["LiangPoly",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Conway_String\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["LiangPoly",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "] calculates Liang  polynomial of a ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " projection given by its Conway symbol and distinguishes amphicheiral  ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " projections with a Liang polynomial satisfying the relationship ",
  StyleBox["L",
    FontSlant->"Italic"],
  "(",
  StyleBox["t",
    FontSlant->"Italic"],
  ")=",
  StyleBox["L",
    FontSlant->"Italic"],
  "(1/",
  StyleBox["t",
    FontSlant->"Italic"],
  ")."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["LinkingNo",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["LinkingNo",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Ul_",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string or Dowker code given as a list."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["LinkingNo",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "] computes the linking number of a  link given by Conway symbol or Dowker \
code."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["MSigRat",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["MSigRat",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Conway_String\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["MSigRat",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "]",
  StyleBox[" ",
    FontWeight->"Bold"],
  "calculates the fraction  and Murasugi signature of a rational KL given by \
its Conway symbol."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["MaxSymmProjAltKL",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["MaxSymmProjAltKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Conway_String\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["MaxSymmProjAltKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "] finds maximum  symmetrical projection of an alternating KL given by its \
Conway symbol."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["MinDowAltKL",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["MinDowAltKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Conway_String\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["MinDowAltKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "] calculates  minimal Dowker code of any alternating ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " given by its Conway symbol."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["MinDowProjAltKL",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["MinDowProjAltKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Ul_",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string,  Dowker code as a list, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  " as a list."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["MinDowProjAltKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "]",
  StyleBox[" ",
    FontWeight->"Bold"],
  "calculates minimal  Dowker code without signs (in ",
  StyleBox["Knotscape",
    FontSlant->"Italic"],
  " form) for a given  alternating ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " projection. An input is its Conway symbol, Dowker code,  or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  "."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["NMoveRat",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["NMoveRat",
    FontWeight->"Bold"],
  "[",
  StyleBox["n_Integer,Conway_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Conway_String\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["NMoveRat",
    FontWeight->"Bold"],
  "[",
  StyleBox["n_Integer,Conway_String",
    FontSlant->"Italic"],
  "] calculates  for a given integer ",
  StyleBox["n",
    FontSlant->"Italic"],
  " and Conway symbol of a ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " the minimum  number of ",
  StyleBox["n",
    FontSlant->"Italic"],
  "-moves neccessary to unknott a minimal projection  of a rational ",
  StyleBox["KL",
    FontSlant->"Italic"],
  "."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["NoSelfCrossNo",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["NoSelfCrossNo",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Ul_",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string,  Dowker code as a list, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  " as a list."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["NoSelfCrossNo",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "] ",
  StyleBox["U",
    FontSlant->"Italic"],
  "-infinity number is the minimum number of  cuts in self-crossing points of \
a ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " necessary to obtain unknot,  or a link without self-crossings. The \
function ",
  StyleBox["NoSelfCrossNo",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "] calculates the ",
  StyleBox["U",
    FontSlant->"Italic"],
  "-infinity number (see: Jablan S.: ",
  StyleBox["Unknotting number  and Infty-Unknotting Number of a Knot",
    FontSlant->"Italic"],
  ", Filomat (Nis), 12:1 (1998),  113-120) of a  ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " given by its Conway symbol, Dowker code, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  "."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["NumberOfKL",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["NumberOfKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Str_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Str_String",
    FontSlant->"Italic"],
  StyleBox["       ",
    FontSlant->"Italic"],
  StyleBox["Str",
    FontSlant->"Italic"],
  " is the name of a list of knots and links with a specified number of \
crossings  (for example, ",
  StyleBox["a",
    FontSlant->"Italic"],
  "10 or ",
  StyleBox["n",
    FontSlant->"Italic"],
  "10,  where ",
  StyleBox["a",
    FontSlant->"Italic"],
  " stands for alternating and                                                \
                                                                     ",
  StyleBox["n",
    FontSlant->"Italic"],
  " for non-alternating knots and links, written as a string.     "
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["NumberOfKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Str_String",
    FontSlant->"Italic"],
  "]",
  StyleBox[" ",
    FontWeight->"Bold"],
  "gives the number of alternating or  non-alternating knots and links with a \
specified number of  crossings."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["PeriodAltKL",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["PeriodAltKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Conway_String\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["PeriodAltKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "] calculates the  period of a given alternating ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " given by its Conway symbol."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["PeriodProjAltKL",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["PeriodProjAltKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Ul_",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string,  Dowker code as a list, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  " as a list."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["PeriodProjAltKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "] calculates the period of a given projection of an alternating knot given \
by its  Conway symbol, Dowker code, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  "."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["RatGenSourKL",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["RatGenSourKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["n_Integer, m_Integer",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["n_Integer, m_Integer",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  StyleBox["n",
    FontSlant->"Italic"],
  " is the number of crossings, and ",
  StyleBox["m",
    FontSlant->"Italic"],
  " is a length of digon chains. "
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["RatGenSourKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["n_Integer, m_Integer",
    FontSlant->"Italic"],
  "] calculates the  number and Conway symbols of all rational generating ",
  StyleBox["KL",
    FontSlant->"Italic"],
  "s (for ",
  StyleBox["m",
    FontSlant->"Italic"],
  "=3)  and rational source ",
  StyleBox["KL",
    FontSlant->"Italic"],
  "s (for ",
  StyleBox["m",
    FontSlant->"Italic"],
  "=2) with ",
  StyleBox["n",
    FontSlant->"Italic"],
  " crossings."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["RatKnotGenU0",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["RatKnotGenU0",
    FontWeight->"Bold"],
  "[",
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "Number of crossings."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["RatKnotGenU0",
    FontWeight->"Bold"],
  "[",
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  "] gives Conway symbols  of all rational links with the unlinking number 0 \
with ",
  StyleBox["n",
    FontSlant->"Italic"],
  " crossings."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["RatKnotGenU1",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["RatKnotGenU1",
    FontWeight->"Bold"],
  "[",
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "Number of crossings."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["RatKnotGenU1",
    FontWeight->"Bold"],
  "[",
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  "] gives Conway symbols  of all rational knots with the unknotting number 1 \
with ",
  StyleBox["n",
    FontSlant->"Italic"],
  " crossings."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["RatLinkU0",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["RatLinkU0",
    FontWeight->"Bold"],
  "[",
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "Number of crossings."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["RatLinkU0",
    FontWeight->"Bold"],
  "[",
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  "] calculates the number  and Conway symbols of all rational unlinks (",
  StyleBox["U",
    FontSlant->"Italic"],
  "=0) with ",
  StyleBox["n",
    FontSlant->"Italic"],
  " crossings."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["RatLinkU1",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["RatLinkU1",
    FontWeight->"Bold"],
  "[",
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "Number of crossings."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["RatLinkU1",
    FontWeight->"Bold"],
  "[",
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  "] calculates the number  and Conway symbols of all rational links with the \
unlinking  number 1, with ",
  StyleBox["n",
    FontSlant->"Italic"],
  " crossings.\""
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["RatReduce",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["RatReduce",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Conway_String\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["RatReduce",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "] reduces Conway symbol of a rational",
  StyleBox[" KL",
    FontSlant->"Italic"],
  "."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["RatSourceKLNo",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["RatSourceKLNo",
    FontWeight->"Bold"],
  "[",
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "Number of crossings."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["RatSourceKLNo",
    FontWeight->"Bold"],
  "[",
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  "] calculates the  number of rational source ",
  StyleBox["KL",
    FontSlant->"Italic"],
  "s (for ",
  StyleBox["k",
    FontSlant->"Italic"],
  "=2) with ",
  StyleBox["n",
    FontSlant->"Italic"],
  " crossings according to the general recursion formula: \n",
  StyleBox["b",
    FontSlant->"Italic"],
  "[0]=1, ",
  StyleBox["b",
    FontSlant->"Italic"],
  "[1]=1, ",
  StyleBox["b",
    FontSlant->"Italic"],
  "[2",
  StyleBox["n",
    FontSlant->"Italic"],
  "-2]+",
  StyleBox["b",
    FontSlant->"Italic"],
  "[2",
  StyleBox["n",
    FontSlant->"Italic"],
  "-1]=",
  StyleBox["b",
    FontSlant->"Italic"],
  "[2",
  StyleBox["n",
    FontSlant->"Italic"],
  "], ",
  StyleBox["b",
    FontSlant->"Italic"],
  "[2",
  StyleBox["n",
    FontSlant->"Italic"],
  "]+",
  StyleBox["b",
    FontSlant->"Italic"],
  "[2",
  StyleBox["n",
    FontSlant->"Italic"],
  "-1]-",
  StyleBox["f",
    FontSlant->"Italic"],
  "[",
  StyleBox["n",
    FontSlant->"Italic"],
  "-1]=",
  StyleBox["b",
    FontSlant->"Italic"],
  "[2",
  StyleBox["n",
    FontSlant->"Italic"],
  "+1], where  ",
  StyleBox["f",
    FontSlant->"Italic"],
  " is the Fibonacci sequence given by recursion  \n",
  StyleBox["f",
    FontSlant->"Italic"],
  "[0]=1, ",
  StyleBox["f",
    FontSlant->"Italic"],
  "[1]=1, ",
  StyleBox["f",
    FontSlant->"Italic"],
  "[",
  StyleBox["n",
    FontSlant->"Italic"],
  "-2]+",
  StyleBox["f",
    FontSlant->"Italic"],
  "[",
  StyleBox["n",
    FontSlant->"Italic"],
  "-1]=",
  StyleBox["f",
    FontSlant->"Italic"],
  "[",
  StyleBox["n",
    FontSlant->"Italic"],
  "]."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["RationalKL",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["RationalKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "Number of crossings."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["RationalKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  "] calculates the number and Conway symbols of all rational",
  StyleBox[" KL",
    FontSlant->"Italic"],
  "s for a given number of crossings ",
  StyleBox["n",
    FontSlant->"Italic"],
  "."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["RationalAmphiK",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["RationalAmphiK",
    FontWeight->"Bold"],
  "[",
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "Number of crossings."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["RationalAmphiK",
    FontWeight->"Bold"],
  "[",
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  "] calculates the number and Conway symbols of all rational amphicheiral \
knots for a given  number of crossings ",
  StyleBox["n",
    FontSlant->"Italic"],
  "."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["SameAltConKL",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["SameAltConKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Con1_String,Con2_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Con1_String,",
    FontSlant->"Italic"],
  StyleBox["Con2_String",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["SameAltConKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Con1_String,Con2_String",
    FontSlant->"Italic"],
  "] compares two alternating ",
  StyleBox["KL",
    FontSlant->"Italic"],
  "s given by Conway symbols. The result is 1 for equal, and 0 for non-equal \
",
  StyleBox["KL",
    FontSlant->"Italic"],
  "s."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["SameAltProjKL",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["SameAltProjKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Con1_String,Con2_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Con1_String,",
    FontSlant->"Italic"],
  StyleBox["Con2_String",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["SameAltProjKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Con1_String,Con2_String",
    FontSlant->"Italic"],
  "] compares two  alternating KL projections given by Conway symbols,  \
Dowker codes, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  ". The result is 1 for equal, and 0 for non-equal projections."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["SplittNo",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["SplittNo",
    FontWeight->"Bold"],
  "[",
  StyleBox["PData_",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["PData_",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "Conway symbol, Dowker code, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  "."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["SplittNo",
    FontWeight->"Bold"],
  "[",
  StyleBox["PData_",
    FontSlant->"Italic"],
  "] calculates splitting number of a link given by its Conway symbol, Dowker \
code, or",
  StyleBox[" pdata",
    FontWeight->"Bold"],
  ", i.e.,  the minimum number of crossing changes necessary in order to \
obtain a splitted  link. It is illustrated by the example from Adams. (see: \
Adams C:  ",
  StyleBox["Splitting Versus Unlinking",
    FontSlant->"Italic"],
  ", Journal of Knot Theory and Ramifications,  Vol.5, No. 3 (1996) \
295-299)."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Symm",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Symm",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Ul_",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "Conway symbol, Dowker code, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  "."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Symm",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "] calculates all automorphisms of an alternating ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " given by its Conway symbol, Dowker code,  or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  ". The first part of the result is the list of  automorphisms given by \
permutations, and the other the  list of the corresponding cycles."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["UnKnotLink",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["UnKnotLink",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Conway_String\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["UnKnotLink",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "] returns an exact or estimated unknotting or unlinking number (",
  StyleBox["U",
    FontSlant->"Italic"],
  " or ",
  StyleBox["U?",
    FontSlant->"Italic"],
  ") of a given knot or link. For the reduction of knots or links it uses ",
  StyleBox["ReductionKnotLink",
    FontWeight->"Bold"],
  "."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["External Program", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  "For the reduction of knots or links by ",
  StyleBox["ReductionKnotLink ",
    FontWeight->"Bold"],
  "it uses ",
  StyleBox["KnotLinkReduction.exe",
    FontWeight->"Bold"],
  "."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Notes", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["* If the estimated unknotting number",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[",",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" computed according to ",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["Bernhard-Jablan Conjecture",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0]],
  StyleBox[": (http://members.tripod.com/vismath/sl/index.html) is 1 or 2, we \
are sure that we obtained the correct unknotting number, and the result is ",
    FontColor->RGBColor[0.500008, 0, 0]],
  StyleBox["U",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0]],
  StyleBox[". Otherwise, the result is an estimated unknotting or unlinking \
number ",
    FontColor->RGBColor[0.500008, 0, 0]],
  StyleBox["U",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0]],
  StyleBox["?.",
    FontColor->RGBColor[0.500008, 0, 0]]
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["UnR",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["UnR",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Conway_String\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["UnR",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "] returns an estimated unknotting or unlinking number ",
  StyleBox["U",
    FontSlant->"Italic"],
  " of a given rational knot or link, computed by using continued fractions."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["External Program", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell["None.", "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Notes", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  "*",
  StyleBox[" An estimated unknotting or unlinking number ",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["U",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" of a rational knot or link ",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["K",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" is computed according to ",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["Bernhard-Jablan Conjecture",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0]],
  StyleBox[" (http://members.tripod.com/vismath/sl/index.html) by using \
continued fractions.",
    FontColor->RGBColor[0.500008, 0, 0]]
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["AmphiAltKL",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["AmphiAltKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Conway_String\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["AmphiAltKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "] tests the  amphicheirality of an alternating ",
  StyleBox["KL ",
    FontSlant->"Italic"],
  "given by its Conway symbol."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["AmphiProjAltKL",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["AmphiProjAltKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Ul_\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string,  Dowker code as a list, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  " as a list."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["AmphiProjAltKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "]",
  StyleBox[" ",
    FontWeight->"Bold"],
  "tests the amphicheirality of  a given projection of an alternating ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " givenby its Conway symbol,  Dowker code, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  "."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fAddDig",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fAddDig",
    FontWeight->"Bold"],
  "[",
  StyleBox["GL_List",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["GL_List",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "Graph given by incidence list."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fAddDig",
    FontWeight->"Bold"],
  "[",
  StyleBox["GL_List",
    FontSlant->"Italic"],
  "] produces from a given graph  all 4-regular non-isomorphic graphs by \
replacing single edges by double (digonal) edges."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fAlexPoly",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fAlexPoly",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Conway_String\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fAlexPoly",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "] calculates multi-variable  Alexander polynomial of a ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " from its Conway symbol  by using Writinger presentation (generators of ",
  StyleBox["KL",
    FontSlant->"Italic"],
  ")."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fAllStatesProj",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fAllStatesProj",
    FontWeight->"Bold"],
  "[",
  StyleBox["UL_",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["UL_\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string,  Dowker code as a list, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  " as a list."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fAllStatesProj",
    FontWeight->"Bold"],
  "[",
  StyleBox["UL_",
    FontSlant->"Italic"],
  "] calculates all states of  a given alternating ",
  StyleBox["KL",
    FontSlant->"Italic"],
  ", i.e., all different variations of signs in  a given ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " projection. Projection can be given by its Conway symbol,  Dowker code, \
or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  ". The result is the list of reduced ",
  StyleBox["KL ",
    FontSlant->"Italic"],
  "obtained  from it by all sign-changes. The first datum is  of crossing \
changes,  and the second the list of all ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " obtained."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fAutoKL",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fAutoKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_List",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Ul_List\t\t",
    FontSlant->"Italic"],
  StyleBox["KL",
    FontSlant->"Italic"],
  " given in Conway  notation, followed by a list of signs of vertices, and  \
inputs in vertices."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fAutoKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_List",
    FontSlant->"Italic"],
  "] calculates the stable  states of an automaton obtained from a ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " given in Conway  notation, followed by a list of signs of vertices, and  \
inputs in vertices. In the vertices with a sign 1 is  used the operation ",
  StyleBox["NOR",
    FontSlant->"Italic"],
  ", and in the vertices -1 the  operation ",
  StyleBox["OR",
    FontSlant->"Italic"],
  ". The result is the list of edge colorings corresponding to stable states \
and the list of stable states  according to vertices (see Kauffman). If the \
signlist  is empty, it is computed as the signlist of the given ",
  StyleBox["KL",
    FontSlant->"Italic"],
  "."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fAutoSigInp",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fAutoSigInp",
    FontWeight->"Bold"],
  "[",
  StyleBox["Gr_List",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Gr_List",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "A list of outgoing edges, signs of vertices and inputs in vertices."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fAutoSigInp",
    FontWeight->"Bold"],
  "[",
  StyleBox["Gr_List",
    FontSlant->"Italic"],
  "] calculates stable states of an automaton, given by a list of outgoing \
edges, signs of vertices and inputs in vertices. In the vertices  with a sign \
1 is used the operation ",
  StyleBox["NOR",
    FontSlant->"Italic"],
  ", and in the  vertices -1 the operation ",
  StyleBox["OR",
    FontSlant->"Italic"],
  ". The result is the list of  edge colorings corresponding to stable states \
and the  list of stable states according to vertices (see Kauffman). If the \
signlist is empty, it is computed as {1,...,1},  i.e. with ",
  StyleBox["NOR",
    FontSlant->"Italic"],
  " in all vertices."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fBalanced",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fBalanced",
    FontWeight->"Bold"],
  "[",
  StyleBox["Diff_List,redBr_Integer",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Diff_List,redBr_Integer",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "A periodic sequence from the list of different periodic sequences with ",
  StyleBox["n",
    FontSlant->"Italic"],
  " variables, of period ",
  StyleBox["p",
    FontSlant->"Italic"],
  ", given by its ordering number."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fBalanced",
    FontWeight->"Bold"],
  "[",
  StyleBox["Diff_List,redBr_Integer",
    FontSlant->"Italic"],
  "]",
  StyleBox[" ",
    FontWeight->"Bold"],
  "calculates stable (balanced) states for a given periodic sequence."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fBasicPoly",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fBasicPoly",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_List",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Ul_List\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string,  Dowker code as a list, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  " as a list."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fBasicPoly",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_List",
    FontSlant->"Italic"],
  "] finds the basic polyhedron for a ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " given by Dowker code, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  "."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fBoolean",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fBoolean",
    FontWeight->"Bold"],
  "[",
  StyleBox["LL_",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["LL_",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "A logical term."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fBoolean",
    FontWeight->"Bold"],
  "[",
  StyleBox["LL_",
    FontSlant->"Italic"],
  "]",
  StyleBox[" ",
    FontWeight->"Bold"],
  "calculates the square-free  polynomial for a given logical term."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fBreakComp",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fBreakComp",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_,k_Integer",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Ul_,k_Integer\t",
    FontSlant->"Italic"],
  StyleBox["\t",
    FontSlant->"Italic"],
  "Conway symbol, Dowker code, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  ", and an ordering number  ",
  StyleBox["k",
    FontSlant->"Italic"],
  " of a cutted component "
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fBreakComp",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_,k_Integer",
    FontSlant->"Italic"],
  "] in a link given by Conway symbol, Dowker code, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  ", and by ordering number  ",
  StyleBox["k",
    FontSlant->"Italic"],
  " of a cutted component calculates the pdata of a link with  ",
  StyleBox["k",
    FontSlant->"Italic"],
  "-th component cutted."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fColTest",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fColTest",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_,cNo_Integer",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Ul_,cNo_Integer",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "Conway symbol, Dowker code, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  ", and a number of colors  (greater then 2)."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fColTest",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_,cNo_Integer",
    FontSlant->"Italic"],
  "]",
  StyleBox[" ",
    FontWeight->"Bold"],
  "calculates from a given Conway symbol, Dowker code,  ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  ", and from a number of colors  (greater then 2), a coloring of ",
  StyleBox["KL",
    FontSlant->"Italic"],
  ". The result is the list of colored  generators according to the generator \
list given by  ",
  StyleBox["fGenerators",
    FontWeight->"Bold"],
  ". The resuct can be a perfect coloring, where in every crossing all  \
generators have different color, standard coloring where all generators  have \
different colors or the same color, and Kauffman coloring where  the only \
forbiden case is that incomming and passing generator, or  outgoing and \
passing generator have the same color, different from  the third generator."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fComponentNo",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fComponentNo",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Ul_\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string,  Dowker code as a list, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  " as a list."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fComponentNo",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "] calculates the number of  components of a ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " given by Conway symbol, Dowker code, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  "."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fCompositePoly",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fCompsitePoly",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Conway_String\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a basic polyhedron as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fCompositePoly",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "] ",
  StyleBox["checks that a basic polyhedron given by its Conway symbol is \
elementary or composite.  The result is 1 for a composite, and 0 for an \
elementary basic polyhedron.",
    FontColor->RGBColor[0.501961, 0, 0]]
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fCreateGraphics",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fCreateGraphics",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Conway_String\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string ",
  StyleBox["K",
    FontSlant->"Italic"],
  "."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fCreateGraphics",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "], by using the external program",
  StyleBox[" DrawKnot.exe",
    FontWeight->"Bold"],
  ", writes in the file ",
  StyleBox["c:\\Program Files\\KnotPlot\\grahics.txt",
    FontWeight->"Bold"],
  "  coordinates of a given knot or link. The file obtained can be loaded in \
the program ",
  StyleBox["KnotPlot",
    FontWeight->"Bold"],
  " (http://www.knotplot.com/download.html) by writing ",
  StyleBox["load graphics.txt ",
    FontWeight->"Bold"],
  "in ",
  StyleBox["KnotPlot",
    FontWeight->"Bold"],
  " command line."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["External Program", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell["DrawKnot.exe", "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Notes", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  "* For viewing the produced file ",
  StyleBox["graphics.txt",
    FontWeight->"Bold"],
  " located in the directory ",
  StyleBox["c:\\Program Files\\KnotPlot\\",
    FontWeight->"Bold"],
  " you need to have insttalled the program ",
  StyleBox["KnotPlot",
    FontWeight->"Bold"],
  " (http://www.pims.math.ca/knotplot/download.html) in that directory. \
Please check that the path to ",
  StyleBox["KnotPlot",
    FontWeight->"Bold"],
  " is correct. The file obtained can be loaded in the program ",
  StyleBox["KnotPlot",
    FontWeight->"Bold"],
  " by writing ",
  StyleBox["load graphics.txt ",
    FontWeight->"Bold"],
  "in the ",
  StyleBox["KnotPlot",
    FontWeight->"Bold"],
  " command line."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fCreatePData",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fCreatePData",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Conway_String\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fCreatePData",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "] calculates ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  " for given knot or link ",
  StyleBox["K",
    FontSlant->"Italic"],
  " given in Conway notation. "
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Notes", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  "* ",
  "The first data in the list ",
  StyleBox["Dow",
    FontSlant->"Italic"],
  " are lengths of components, and the second Dowker code with the signs of \
crossing points.",
  StyleBox[" ",
    FontVariations->{"CompatibilityType"->0}],
  "  "
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fCuttRealKL",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fCuttRealKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["UL_",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["UL_\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string,  Dowker code as a list, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  " as a list."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fCuttRealKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["UL_",
    FontSlant->"Italic"],
  "] calculates the number of  real cuttings of a given projection of ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " given by its Conway  symbol, Dowker code, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  ", i.e., the number of cuttings  with a different cutting point in the \
projection. The result is the number of different real cutting classes with \
preserved  signs, or with preserved or reversed signs."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fDToDDirect",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fDToDDirect",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Conway_String\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fDToDDirect",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "] computes Dowker code of a direct product of ",
  StyleBox["KL",
    FontSlant->"Italic"],
  "s given in Conway notation."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fDiffProjectionsAltKL",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fDiffProjectionsAltKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Conway_String\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fDiffProjectionsAltKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "] calculates  Conway symbols and minimal Dowker codes of all \
non-isomorphic projections of a ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " given by its Conway symbol."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fDiffSeq",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fDiffSeq",
    FontWeight->"Bold"],
  "[",
  StyleBox["n_Integer,p_Integer",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["n_Integer,p_Integer",
    FontSlant->"Italic"],
  StyleBox["\t                ",
    FontSlant->"Italic"],
  " ",
  StyleBox["n",
    FontSlant->"Italic"],
  " is a number of variables, and ",
  StyleBox["p",
    FontSlant->"Italic"],
  " a period."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fDiffSeq",
    FontWeight->"Bold"],
  "[",
  StyleBox["n_Integer,p_Integer",
    FontSlant->"Italic"],
  "] calculates  all different periodic sequences with ",
  StyleBox["n",
    FontSlant->"Italic"],
  " variables, of period ",
  StyleBox["p",
    FontSlant->"Italic"],
  ". As the result, every sequence is given  by its code, Kauffman code, and \
square-free polynomials  corresponding to it."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fDowfromPD",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fDowfromPD",
    FontWeight->"Bold"],
  "[",
  StyleBox["pdata_List",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["pdata_List",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  StyleBox["pdata",
    FontWeight->"Bold"],
  " given as a list."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fDowfromPD",
    FontWeight->"Bold"],
  "[",
  StyleBox["pdata_List",
    FontSlant->"Italic"],
  "] computes Dowker code from pdata."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fGenSign",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fGenSign",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Conway_String\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string.\t\t"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fGenSign",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "] returns the signlist of crossing points of a knot or link."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["External program", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell["None.", "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fDowfromPD",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fDowfromPD",
    FontWeight->"Bold"],
  "[",
  StyleBox["pdata_List",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["pdata_List",
    FontSlant->"Italic"],
  "\t\tP-data in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " is a list having two entries which are also lists. The first one \
indicates the number of crossings of each component, and the second is number \
sequence derived from the knot (or link), called P-word."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fDowfromPD",
    FontWeight->"Bold"],
  "[",
  StyleBox["pdata_List",
    FontSlant->"Italic"],
  "] computes Dowker code ",
  StyleBox["Dow",
    FontSlant->"Italic"],
  " from ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  "."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["External Program", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell["None.", "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Notes", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  "* The first data of the list ",
  StyleBox["Dow",
    FontSlant->"Italic"],
  " obtained as a result are lengths of components (the number of crossings \
of each component), and the second Dowker code (number sequence of even \
numbers derived from the knot or link with the signs of crossing points).",
  StyleBox[" ",
    FontVariations->{"CompatibilityType"->0}]
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fEdmonds",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fEdmonds",
    FontWeight->"Bold"],
  "[",
  StyleBox["GG_List",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["GG_List",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "A graph of a ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " given as a list of  unordered pairs."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fEdmonds",
    FontWeight->"Bold"],
  "[",
  StyleBox["GG_List",
    FontSlant->"Italic"],
  "] gives list of polygons, characteristics and genus for a graph of a ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " given as a list of  unordered pairs."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fFindCon",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fFindCon",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Ul_\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string,  Dowker code as a list, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  " as a list."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fFindCon",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "] finds Conway symbol of  any alternating ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " with at most 12 vertices  given by its Dowker code with signs, or by ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  "."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fGap",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fGap",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Conway_String\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fGap",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "] finds ",
  StyleBox["KL",
    FontSlant->"Italic"],
  "s with the unlinking gap. The first part of result is 1 for ",
  StyleBox["KL",
    FontSlant->"Italic"],
  "s with an unlinking gap,  and 0 otherwise, and second part is unlinking \
number."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fGapRat",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fGapRat",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Conway_String\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fGapRat",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "] finds rational ",
  StyleBox["KL",
    FontSlant->"Italic"],
  "s with an unlinking gap. The result is the unlinking gap, followed by  the \
unlinking number of ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " and the unlinking number of its fixed minimal projection."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fGaussExtSigns",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fGaussExtSigns",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ulaz_",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Ulaz_\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string,  Dowker code as a list, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  " as a list."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fGaussExtSigns",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ulaz_",
    FontSlant->"Italic"],
  "] calculates Gauss code with signs for a link projection given by its \
Conway symbol, Dowker code, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  "."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fGenSign",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fGenSign",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Conway_String\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fGenSign",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "] returns the signlist of crossing points of a ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " given by Conway symbol."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fGenSignDirProd",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fGenSignDirProd",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Conway_String\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fGenSignDirProd",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "]",
  StyleBox[" ",
    FontWeight->"Bold"],
  "computes signs of a direct product of ",
  StyleBox["KL",
    FontSlant->"Italic"],
  "s given in Conway notation."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fGenerators",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fGenerators",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Ul_\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string,  Dowker code as a list, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  " as a list."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fGenerators",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "] calculates generators of a",
  StyleBox[" KL",
    FontSlant->"Italic"],
  " and the list of the signs corresponding to crossing points for a ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " given by Conway symbol,  Dowker code, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  ". The first generator incommes to  a vertex, the seccond is a the passing \
generator, and the  third is the outgoing generator \
(IPO=Incomming-Passing-Outgoing).  The result is divided according to the \
components of ",
  StyleBox["KL",
    FontSlant->"Italic"],
  "."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fGraphInc",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fGraphInc",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Ul_\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string,  Dowker code as a list, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  " as a list."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fGraphInc",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "] calculates from a Conway symbol, Dowker code, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  " of a ",
  StyleBox["KL",
    FontSlant->"Italic"],
  ", the corresponding graph of ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " given by edges (unordered pairs),  and by the list of vertex signs."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fGraphKL",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fGraphKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ulaz_",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Ulaz_\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string,  Dowker code as a list, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  " as a list."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fGraphKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ulaz_",
    FontSlant->"Italic"],
  "] calculates and draws graph of a ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " given by Conway symbol, Dowker code, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  ". The result is the graph given by the list of unordered pairs."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fJablanPoly",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fJablanPoly",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Conway_String\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fJablanPoly",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "] calculates multivariable Jablan polynomial for a projection of ",
  StyleBox["KL",
    FontSlant->"Italic"],
  "  given by its Conway symbol."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fKLfromGraph",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fKLfromGraph",
    FontWeight->"Bold"],
  "[",
  StyleBox["L_List",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["L_List\t\t",
    FontSlant->"Italic"],
  "          Graph ",
  StyleBox["G",
    FontSlant->"Italic"],
  " given by a list of unordered pairs of vertices."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fKLfromGraph",
    FontWeight->"Bold"],
  "[",
  StyleBox["L_List",
    FontSlant->"Italic"],
  "] fKLfromGraph[",
  StyleBox["L_List",
    FontSlant->"Italic"],
  "] gives the",
  StyleBox[" KL",
    FontSlant->"Italic"],
  " defined by a graph ",
  StyleBox["G",
    FontSlant->"Italic"],
  " given as a list of unordered pairs."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fKLinGraph",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fKLinGraph",
    FontWeight->"Bold"],
  "[",
  StyleBox["UOPair_List",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["UOPair_List",
    FontSlant->"Italic"],
  StyleBox["\t",
    FontSlant->"Italic"],
  "           A graph ",
  StyleBox["G",
    FontSlant->"Italic"],
  " given by a list of unordered pairs of vertices."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fKLinGraph",
    FontWeight->"Bold"],
  "[",
  StyleBox["UOPair_List",
    FontSlant->"Italic"],
  "] gives all non-isomorphic",
  StyleBox[" KL",
    FontSlant->"Italic"],
  " projections contained in a graph ",
  StyleBox["G",
    FontSlant->"Italic"],
  " given by a list of unordered pairs of vertices."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fKnotscapeDow",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fKnotscapeDow",
    FontWeight->"Bold"],
  "[",
  StyleBox["Con_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Con_String\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fKnotscapeDow",
    FontWeight->"Bold"],
  "[",
  StyleBox["Con_String",
    FontSlant->"Italic"],
  "] calculates from a Conway symbol of a ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " its Dowker code in a the ",
  StyleBox["Knotscape ",
    FontSlant->"Italic"],
  "format:  Dowker code without signs for alternating ",
  StyleBox["KL",
    FontSlant->"Italic"],
  ", or Dowker code with signs of changed crossings for nonalternating ",
  StyleBox["KL",
    FontSlant->"Italic"],
  "."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fMidEdgeGraph",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fMidEdgeGraph",
    FontWeight->"Bold"],
  "[",
  StyleBox["L_List",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["L_List",
    FontSlant->"Italic"],
  StyleBox["\t",
    FontSlant->"Italic"],
  "           A polyhedral graph ",
  StyleBox["G",
    FontSlant->"Italic"],
  " given by a list of unordered pairs of vertices."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fMidEdgeGraph",
    FontWeight->"Bold"],
  "[",
  StyleBox["L_List",
    FontSlant->"Italic"],
  "] gives the graph defined by mid-edge points of a polyhedral graph ",
  StyleBox["G",
    FontSlant->"Italic"],
  " given by a list of unordered pairs of vertices."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fMillett",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fMillett",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Conway_String\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fMillett",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "] writes in the file ",
  StyleBox["izlaz11.txt",
    FontWeight->"Bold"],
  " Millet code of a knot or link. The external program ",
  StyleBox["lmp1.exe",
    FontWeight->"Bold"],
  " calculates from it polynomial invariants, including Homfly polynomial. \
Millet code obtained is compatible with the program ",
  StyleBox["Knotscape ",
    FontWeight->"Bold"],
  "(http://www.math.utk.edu/~morwen/knotscape.html)."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["External Program", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["lmp.exe",
    FontWeight->"Bold"],
  " which calls",
  StyleBox[" izlaz11.txt",
    FontWeight->"Bold"],
  " and c",
  "alculates from it the polynomial invariants, including Homfly polynomial."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Notes", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  "* The first data in the list ",
  StyleBox["Dow",
    FontSlant->"Italic"],
  " are lengths of components, and the second Dowker code with the signs of \
crossing points.",
  StyleBox[" ",
    FontVariations->{"CompatibilityType"->0}],
  "\n* ",
  StyleBox["lmp1.exe",
    FontWeight->"Bold"],
  " need to be placed in the directory ",
  StyleBox["K2K",
    FontWeight->"Bold"],
  ".\n* ",
  StyleBox["lmp1.exe",
    FontWeight->"Bold"],
  " calculates ",
  StyleBox["H",
    FontSlant->"Italic"],
  "-polynomial (Homfly polynomial), Jones polynomial, Alexander polynomial \
and vals.",
  StyleBox[" ",
    FontVariations->{"CompatibilityType"->0}],
  "  "
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fOrientedLink",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fOrientedLink",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Ul_\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string,  Dowker code as a list, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  " as a list."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fOrientedLink",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "] calculates Gauss codes for a link projection given by Conway symbol,  \
Dowker code, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  ". The Gauss codes obtained correspond  to different orientations of \
components (the first part of data  obtained). The second part gives the \
orientations of components  (where + is denoted by 1, and - by 0), and the \
third part is the  (signed) linking number of the corrsponding oriented \
link."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fPDataFromDow",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fPDataFromDow",
    FontWeight->"Bold"],
  "[",
  StyleBox["Dow_List",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Dow_List",
    FontSlant->"Italic"],
  StyleBox["                   ",
    FontSlant->"Italic"],
  "Dowker code in ",
  StyleBox["Knotscape",
    FontSlant->"Italic"],
  " format (without signs for an alternating prime ",
  StyleBox["KL",
    FontSlant->"Italic"],
  ", or from Dowker code with signs of changed  crossings for    \
nonalternating prime ",
  StyleBox["KL",
    FontSlant->"Italic"],
  "), given as a list."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fPDataFromDow",
    FontWeight->"Bold"],
  "[",
  StyleBox["Dow_List",
    FontSlant->"Italic"],
  "] calculates ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  " from Dowker code in ",
  StyleBox["Knotscape",
    FontSlant->"Italic"],
  " format (without signs for an alternating prime ",
  StyleBox["KL",
    FontSlant->"Italic"],
  ", or from Dowker code with signs of changed  crossings for nonalternating \
prime KL)."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fPDataFromDowker",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fPDataFromDowker",
    FontWeight->"Bold"],
  "[",
  StyleBox["Dow_List",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Dow_List                   ",
    FontSlant->"Italic"],
  "Dowker code with signs given as a list."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fPDataFromDowker",
    FontWeight->"Bold"],
  "[",
  StyleBox["Dow_List",
    FontSlant->"Italic"],
  "] calculates ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  " from Dowker code without signs."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fPlanarEmb",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fPlanarEmb",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Ul_\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string,  Dowker code as a list, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  " as a list."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["External Program", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell["planarity.exe", "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fPlanarEmb",
    FontWeight->"Bold"],
  "[",
  StyleBox["neu_List",
    FontSlant->"Italic"],
  "] calculates the planar embedding of a prime ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " given by Conway symbol, ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  " or  Dowker code. An output is the list that consists of the graph of  \
input ",
  StyleBox["KL",
    FontSlant->"Italic"],
  ", its planar embedding given  by vertex cycles, and faces of planar \
embedded graph. As the basis of  this program it is used the external MS DOS \
program ",
  StyleBox["planarity.exe",
    FontSlant->"Italic"],
  " written by J.M.Boyer."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fPlanarEmbGraph",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fPlanarEmbGraph",
    FontWeight->"Bold"],
  "[",
  StyleBox["LP_",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["LP_",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "A 3-connected planar graph given by a list of unordered pairs of \
vertices."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["External Program", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell["planarity.exe", "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fPlanarEmbGraph",
    FontWeight->"Bold"],
  "[",
  StyleBox["LP_",
    FontSlant->"Italic"],
  "] gives the planar embedding  of a 3-connected planar graph given by a \
list of unordered pairs. An output is the list that consists of the graph \
given by a list of unordered pairs, its planar embedding given by vertex \
cycles,  and faces of planar embedded graph. As the basis of  this program it \
is used the external MS DOS program ",
  StyleBox["planarity.exe",
    FontSlant->"Italic"],
  " written by J.M.Boyer."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fPolyFlype",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fPolyFlype",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Conway_String\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a basic polyhedron as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fPolyFlype",
    FontWeight->"Bold"],
  "[",
  StyleBox["Conway_String",
    FontSlant->"Italic"],
  "] ",
  StyleBox["checks that a basic polyhedron given by its Conway symbol permits \
flypes or not.  The result is 1 for basic polyhedra permiting flypes, and 0 \
for the others. ",
    FontColor->RGBColor[0.501961, 0, 0]]
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fPrimeGraph",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fPrimeGraph",
    FontWeight->"Bold"],
  "[",
  StyleBox["GG_List",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["GG_List",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "A graph given by a list of unordered pairs of vertices."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fPrimeGraph",
    FontWeight->"Bold"],
  "[",
  StyleBox["GG_List",
    FontSlant->"Italic"],
  "] tests that a graph given by a list of unordered pairs, after \
transforming it into the corresponding alternating ",
  StyleBox["KL",
    FontSlant->"Italic"],
  ", will result in a prime or composite alternating ",
  StyleBox["KL",
    FontSlant->"Italic"],
  ". The output is 1 for a prime,  and 0 for a composite KL."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fPrimeKL",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fPrimeKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Ul_",
    FontSlant->"Italic"],
  StyleBox["      \t",
    FontSlant->"Italic"],
  "        Dowker code as a list, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  " as a list."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fPrimeKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "] checks that ",
  StyleBox["KL ",
    FontSlant->"Italic"],
  "given by Dowker code,  or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  " is a prime or composite ",
  StyleBox["KL",
    FontSlant->"Italic"],
  ", i.e. a direct product of some  prime ",
  StyleBox["KL",
    FontSlant->"Italic"],
  "s. The result is 1 for a prime ",
  StyleBox["KL",
    FontSlant->"Italic"],
  ", and 0 for a composite ",
  StyleBox["KL",
    FontSlant->"Italic"],
  "."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fProjections",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fProjections",
    FontWeight->"Bold"],
  "[",
  StyleBox["Con_String",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Con_String",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fProjections",
    FontWeight->"Bold"],
  "[",
  StyleBox["Con_String",
    FontSlant->"Italic"],
  "] calculates  Conway symbols of all projections of a ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " given by its Conway symbol."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fSeifert",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fSeifert",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Ul_",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string,  Dowker code as a list, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  " as a list."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fSeifert",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "]",
  StyleBox[" ",
    FontWeight->"Bold"],
  "calculates Seifert matrix of a ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " given by its Conway symbol, Dowker code,  or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  ". Its basic function ",
  StyleBox["SeifertMatrix",
    FontWeight->"Bold"],
  "  is written by Stepan Orevkov."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fSignat",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fSignat",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Ul_",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "Conway symbol of a knot or link as a \"",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\" string,  Dowker code as a list, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  " as a list."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fSignat",
    FontWeight->"Bold"],
  "[",
  StyleBox["Ul_",
    FontSlant->"Italic"],
  "]",
  StyleBox[" ",
    FontWeight->"Bold"],
  "calculates signature of a ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " given by its Conway symbol, Dowker code, or ",
  StyleBox["pdata",
    FontWeight->"Bold"],
  ". Its basic function ",
  StyleBox["ssmW",
    FontWeight->"Bold"],
  " is written by Stepan Orevkov."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fSignsKL",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fSignsKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Dow_List",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["Dow_List",
    FontSlant->"Italic"],
  StyleBox["                   ",
    FontSlant->"Italic"],
  "Dowker code in ",
  StyleBox["Knotscape",
    FontSlant->"Italic"],
  " format (without signs for an alternating prime ",
  StyleBox["KL",
    FontSlant->"Italic"],
  ", or from Dowker code with signs of changed  crossings for    \
nonalternating prime ",
  StyleBox["KL",
    FontSlant->"Italic"],
  "), given as a list."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fSignsKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["Dow_List",
    FontSlant->"Italic"],
  "]",
  StyleBox[" ",
    FontWeight->"Bold"],
  "calculates Dowker code with signs of a ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " given by its  Dowker code in ",
  StyleBox["Knotscape",
    FontSlant->"Italic"],
  " form.  For an alternating knot, an input is Dowker code without signs,  \
and for a nonalternatng ",
  StyleBox["KL",
    FontSlant->"Italic"],
  ", an input is Dowker code containing only signs of points with signs \
changed with regard to the  corresponding alternating ",
  StyleBox["KL",
    FontSlant->"Italic"],
  "."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fStellarBasic",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fStellarBasic",
    FontWeight->"Bold"],
  "[",
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "Number of crossings."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fStellarBasic",
    FontWeight->"Bold"],
  "[",
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  "] calculates the number and Conway symbols of all basic stellar",
  StyleBox[" KL",
    FontSlant->"Italic"],
  "s for a given number of crossings ",
  StyleBox["n",
    FontSlant->"Italic"],
  "."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fStellar",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fStellar",
    FontWeight->"Bold"],
  "[",
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "Number of crossings."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fStellar",
    FontWeight->"Bold"],
  "[",
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  "] calculates the number and Conway symbols of all stellar",
  StyleBox[" KL",
    FontSlant->"Italic"],
  "s for a given number of crossings ",
  StyleBox["n",
    FontSlant->"Italic"],
  "."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fStellarPlus",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fStellarPlus",
    FontWeight->"Bold"],
  "[",
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "Number of crossings."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fStellarPlus",
    FontWeight->"Bold"],
  "[",
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  "] calculates the number and Conway symbols of all stellar",
  StyleBox[" KL",
    FontSlant->"Italic"],
  "s with pluses for a given number of crossings ",
  StyleBox["n",
    FontSlant->"Italic"],
  "."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fStellarNalt",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fStellarNalt",
    FontWeight->"Bold"],
  "[",
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  StyleBox["\t\t",
    FontSlant->"Italic"],
  "Number of crossings."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fStellarNalt",
    FontWeight->"Bold"],
  "[",
  StyleBox["n_Integer",
    FontSlant->"Italic"],
  "] calculates the number and Conway symbols of all non-alternating \
stellar",
  StyleBox[" KL",
    FontSlant->"Italic"],
  "s for a given number of crossings ",
  StyleBox["n",
    FontSlant->"Italic"],
  "."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["fTorusKL",
  FontWeight->"Bold"]], "Subsubtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Specification", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fTorusKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["a_Integer,b_Integer",
    FontSlant->"Italic"],
  "]"
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["a_Integer,b_Integer",
    FontSlant->"Italic"],
  StyleBox["                   ",
    FontSlant->"Italic"],
  "Parameters of a torus ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " [",
  StyleBox["a",
    FontSlant->"Italic"],
  ",",
  StyleBox["b",
    FontSlant->"Italic"],
  "]."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]],

Cell[TextData[{
  StyleBox["fTorusKL",
    FontWeight->"Bold"],
  "[",
  StyleBox["a_Integer,b_Integer",
    FontSlant->"Italic"],
  "] calculates for a torus ",
  StyleBox["KL",
    FontSlant->"Italic"],
  " [",
  StyleBox["a",
    FontSlant->"Italic"],
  ",",
  StyleBox["b",
    FontSlant->"Italic"],
  "] its braid word, min. number of crossings, unknotting  number or number \
of components, bridge number, Alexander polynomial  and (Murasugi) signature \
(see: Murasugi K.,",
  StyleBox[" Knot Theory and its Applications",
    FontSlant->"Italic"],
  ", Birkhauser, Boston, Basel, Berlin, 1996)."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Appendix 1 - Notation", "Subtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Conway notation", "Subtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[TextData[{
  StyleBox["Instead of the graphical input, you can use ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["Conway notation",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" of knots and links (Rolfsen D.: ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["Knots and Links.",
    FontFamily->"Arial",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" Publish and Perish, Berkeley, 1976) represented as a ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["Mathematica",
    FontFamily->"Arial",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" string. For example, the knot 4_1 is denoted by \"2 2\", knot \
9_5 by \"5 1 3\",  link 5_1^2 is denoted by \"2 1 2\" , link 9_21^2 by \"3 \
1,3,2\" (all of them with spaces between tangles), ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["etc",
    FontFamily->"Arial",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". A sequence of ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["k",
    FontFamily->"Arial",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" pluses at the end of Conway symbol is denoted by +",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["k",
    FontFamily->"Arial",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", and the sequence of ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["k",
    FontFamily->"Arial",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" minuses by +-",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["k",
    FontFamily->"Arial",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" (e.g., knot 10_76 given in Conway notation as 3,3,2++ is denoted \
by \"3,3,2+2\", and the link 9_17^3 given in Conway notation as 3,2,2,2-- by \
\"3,2,2,2+-2\"). The space is used in the same way in all other symbols. For \
example, the knot 10_133 is denoted by \"2 3,2 1,2+-1\", and the knot 10_154 \
by \"(2 1,2) -(2 1,2)\" (with spaces). For the basic polyhedra with ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["N",
    FontFamily->"Arial",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["<10 crossings  standard notation is used (.1 , 6*, 8*,  9*, where \
the symbol for 6* can be ommitted). For example, the knot 10_95 is denoted by \
\".2 1 0.2.2\", and 10_101 by \"2 1..2..2\". For higher values of ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["N",
    FontFamily->"Arial",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["  it is used notation in which the first number is the number of \
crossings,  and the next is the ordering number of polyhedron  (e.g., 101*, \
102*, 103* for ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["N",
    FontFamily->"Arial",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["=10 denoting 10*, 10**, 10***, respectively, and 111*, 112*, 113* \
for ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["N",
    FontFamily->"Arial",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["=11 denoting 11*, 11**, 11***, respectively, ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["etc",
    FontFamily->"Arial",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[".). The order of  basic polyhedra for ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["N",
    FontFamily->"Arial",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["=12 corresponds to their list made by A.Caudron (Caudron A.: ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["Classification des noeuds et des enlancements.",
    FontFamily->"Arial",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" Public. Math. d'Orsay 82. Orsay: Univ. Paris Sud, Dept. Math., \
1982. ), so as 121* till 1212* are denoted the basic polyhedra originally \
titled as 12A-12L. For ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["N",
    FontFamily->"Arial",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[">12  the database of basic polyhedra is produced from the list of \
",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["simple 4-regular 4-edge-connected but not 3-connected plane \
graphs",
    FontFamily->"Arial"],
  StyleBox[" generated by Brendan McKay using the program \"plantri\" written \
by Gunnar Brinkmann and Brendan McKay (http://cs.anu.edu.au/~bdm/plantri/).  \
You can work directly with the basic polyhedra with maximum 16 crossings from \
the file ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["PolyBase.m",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". For ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["N",
    FontFamily->"Arial",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["=12 it contains 12 basic polyhedra beginning from 121* to 1212*, \
for ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["N",
    FontFamily->"Arial",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["=13 it contains 19 basic polyhedra from 131* to 1319*, for ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["N",
    FontFamily->"Arial",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["=14  it contains 64 basic polyhedra from 141* to 1464, for ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["N",
    FontFamily->"Arial",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["=15 it contains 155 basic polyhedra from 151* to 15155*, and for \
",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["N",
    FontFamily->"Arial",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["=16 it contains  510 basic polyhedra, begining from 161* to \
16510*. In order to work with the basic polyhedra from ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["N",
    FontFamily->"Arial",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["=17 to ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["N",
    FontFamily->"Arial",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["=20 vertices, you need to open an additional database ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["PolyBase",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["N",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[".m",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", for ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["N",
    FontFamily->"Arial",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["=17 to ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["N",
    FontFamily->"Arial",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["=20 (by writing, e.g.  ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["<<PolyBase17.m ",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["or ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["Needs[\"PolyBase17.m\"]",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["  for ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["N",
    FontFamily->"Arial",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["=17). They  containin, respectively, 1514 basic polyhedra for ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["N",
    FontFamily->"Arial",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["=17 from 171* to 171514*, 5146 basic polyhedra for ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["N",
    FontFamily->"Arial",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["=18 from 181* to 185146*, 16966 basic polyhedra for ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["N",
    FontFamily->"Arial",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["=19 from 191* to 1916966*, and 58782 basic polyhedra for ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["N",
    FontFamily->"Arial",
    FontSlant->"Italic",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["=20 from 201* to 2058782*. In the symbols of polyhedral knots and \
links omitted vertices are denoted by series of single dots (without using :, \
:. or ::), and with zeros separated by space. For example, the knot 10_156 is \
denoted by \"-3..2..2 0\" (with a series of single dots and 0 separated by \
space). ",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}],
  Cell[BoxData[GridBox[{
          {
            StyleBox[\(No . \ of\ crossings\),
              FontWeight->"Bold"], 
            StyleBox[\(File\ name\),
              FontWeight->"Bold"], 
            StyleBox["Range",
              FontWeight->"Bold"]},
          {GridBox[{
                {"6"},
                {"8"},
                {"9"},
                {"10"},
                {"11"},
                {"12"},
                {"13"},
                {"14"},
                {"15"},
                {"16"}
                }], \(Polybase . m\), GridBox[{
                {\( .1, \(\(6\)\(*\)\)\)},
                {\(\(8\)\(*\)\)},
                {\(\(9\)\(*\)\)},
                {\(from\ \ 101*\ to\ 103*\)},
                {\(from\ 111*\ to\ 113*\)},
                {\(from\ 121*\ to\ 1212*\)},
                {\(\(from\)\(\ \ \)\(131\)\(*\)\(to\)\(\ \)\(1319\)\(*\)\(\ \
\)\)},
                {\(from\ 141*to\ 1464*\)},
                {\(from\ 151*\ to\ 15155*\)},
                {\(from\ 161*\ to\ 16510*\)}
                }]},
          {"17", \(PolyBase17 . m\), \(from\ 171*to\ 171514*\)},
          {"18", \(PolyBase18 . m\), \(from\ 181*to\ 185146*\)},
          {"19", \(PolyBase19 . m\), \(from\ 191*to\ 1916966*\)},
          {"20", \(PolyBase20 . m\), \(from\ 201*to\ 2058782*\)}
          },
        GridFrame->True,
        RowLines->True,
        ColumnLines->True]],
    FontFamily->"Arial"],
  StyleBox["\n",
    FontFamily->"Arial",
    FontColor->RGBColor[0.500008, 0, 0],
    FontVariations->{"CompatibilityType"->0}]
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Appendix 2 - Confirmation of functions and external programs \
\>", "Subtitle",
  CellFrame->True,
  FontColor->RGBColor[0.500008, 0, 0],
  Background->GrayLevel[0.849989]],

Cell[TextData[{
  "This table confirms the names of functions and the corresponding external \
programs. All of these external programs need to be placed in ",
  StyleBox["K2K",
    FontWeight->"Bold"],
  ". \n",
  Cell[BoxData[GridBox[{
          {
            StyleBox[\(Functions\ in\ Mathematica\),
              FontFamily->"Arial",
              FontWeight->"Bold"], 
            StyleBox[\(External\ Programs\),
              FontFamily->"Arial",
              FontWeight->"Bold"]},
          {
            StyleBox[
              RowBox[{
                StyleBox["fCreatePData",
                  FontFamily->"Arial",
                  FontWeight->"Bold"], 
                StyleBox["[",
                  FontFamily->"Arial",
                  FontWeight->"Bold"], 
                StyleBox["Conway_String",
                  FontFamily->"Arial",
                  FontWeight->"Plain",
                  FontSlant->"Italic",
                  FontTracking->"Plain",
                  FontVariations->{"Underline"->False,
                  "Outline"->False,
                  "Shadow"->False,
                  "StrikeThrough"->False,
                  "Masked"->False,
                  "CompatibilityType"->0,
                  "RotationAngle"->0}], 
                StyleBox["]",
                  FontFamily->"Arial",
                  FontWeight->"Bold"]}],
              FontFamily->"Arial"], 
            StyleBox["None",
              FontFamily->"Arial"]},
          {
            StyleBox[
              RowBox[{
                StyleBox["Dow",
                  FontFamily->"Arial",
                  FontWeight->"Bold"], 
                StyleBox["[",
                  FontFamily->"Arial",
                  FontWeight->"Bold"], 
                StyleBox["Conway_String",
                  FontFamily->"Arial",
                  FontSlant->"Italic",
                  FontVariations->{"CompatibilityType"->0}], 
                StyleBox["]",
                  FontFamily->"Arial",
                  FontWeight->"Bold"]}],
              FontFamily->"Arial"], 
            StyleBox["None",
              FontFamily->"Arial"]},
          {
            StyleBox[
              RowBox[{
                StyleBox["fGenSign",
                  FontFamily->"Arial",
                  FontWeight->"Bold"], 
                StyleBox["[",
                  FontFamily->"Arial",
                  FontWeight->"Bold"], 
                StyleBox["Conway_String",
                  FontFamily->"Arial",
                  FontWeight->"Plain",
                  FontSlant->"Italic",
                  FontTracking->"Plain",
                  FontVariations->{"Underline"->False,
                  "Outline"->False,
                  "Shadow"->False,
                  "StrikeThrough"->False,
                  "Masked"->False,
                  "CompatibilityType"->0,
                  "RotationAngle"->0}], 
                StyleBox["]",
                  FontFamily->"Arial",
                  FontWeight->"Bold"]}],
              FontFamily->"Arial"], 
            StyleBox["None",
              FontFamily->"Arial"]},
          {
            StyleBox[
              RowBox[{
                StyleBox["fDowfromPD",
                  FontFamily->"Arial",
                  FontWeight->"Bold"], 
                StyleBox["[",
                  FontFamily->"Arial",
                  FontWeight->"Bold"], 
                StyleBox["pdata_List",
                  FontFamily->"Arial",
                  FontWeight->"Plain",
                  FontSlant->"Italic",
                  FontTracking->"Plain",
                  FontVariations->{"Underline"->False,
                  "Outline"->False,
                  "Shadow"->False,
                  "StrikeThrough"->False,
                  "Masked"->False,
                  "CompatibilityType"->0,
                  "RotationAngle"->0}], 
                StyleBox["]",
                  FontFamily->"Arial",
                  FontWeight->"Bold"]}],
              FontFamily->"Arial"], 
            StyleBox["None",
              FontFamily->"Arial"]},
          {
            StyleBox[
              RowBox[{
                StyleBox["fMillett",
                  FontFamily->"Arial",
                  FontWeight->"Bold"], 
                StyleBox["[",
                  FontFamily->"Arial",
                  FontWeight->"Bold"], 
                StyleBox["Conway_String",
                  FontFamily->"Arial",
                  FontWeight->"Plain",
                  FontSlant->"Italic",
                  FontTracking->"Plain",
                  FontVariations->{"Underline"->False,
                  "Outline"->False,
                  "Shadow"->False,
                  "StrikeThrough"->False,
                  "Masked"->False,
                  "CompatibilityType"->0,
                  "RotationAngle"->0}], 
                StyleBox["]",
                  FontFamily->"Arial",
                  FontWeight->"Bold"]}],
              FontFamily->"Arial"], 
            StyleBox[
              RowBox[{
                StyleBox[\(lmp1 . exe\),
                  FontFamily->"Arial"], 
                StyleBox[" ",
                  FontFamily->"Arial"], 
                StyleBox["*",
                  FontFamily->"Arial",
                  FontWeight->"Bold"], 
                StyleBox["1",
                  FontFamily->"Arial",
                  FontWeight->"Bold"]}],
              FontFamily->"Arial"]},
          {
            StyleBox[
              RowBox[{
                StyleBox["NumberOfKL",
                  FontWeight->"Bold"], 
                StyleBox["[",
                  FontFamily->"Arial",
                  FontWeight->"Bold"], "Str_String", 
                StyleBox["]",
                  FontFamily->"Arial",
                  FontWeight->"Bold"]}],
              FontFamily->"Arial"], 
            StyleBox["None",
              FontFamily->"Arial"]},
          {
            StyleBox[
              RowBox[{
                StyleBox["GetKnotlink",
                  FontWeight->"Bold"], 
                StyleBox["[",
                  FontWeight->"Bold"], 
                StyleBox[\(Str_String, No_Integer\),
                  FontWeight->"Plain",
                  FontSlant->"Plain",
                  FontTracking->"Plain",
                  FontVariations->{"Underline"->False,
                  "Outline"->False,
                  "Shadow"->False,
                  "StrikeThrough"->False,
                  "Masked"->False,
                  "CompatibilityType"->0,
                  "RotationAngle"->0}], 
                StyleBox["]",
                  FontWeight->"Bold"]}],
              FontFamily->"Arial"], 
            StyleBox["None",
              FontFamily->"Arial"]},
          {
            StyleBox[
              RowBox[{
                StyleBox["UnKnotLink",
                  FontFamily->"Arial",
                  FontWeight->"Bold"], 
                StyleBox["[",
                  FontFamily->"Arial",
                  FontWeight->"Bold"], 
                StyleBox["Conway_String",
                  FontFamily->"Arial",
                  FontWeight->"Plain",
                  FontSlant->"Italic",
                  FontTracking->"Plain",
                  FontVariations->{"Underline"->False,
                  "Outline"->False,
                  "Shadow"->False,
                  "StrikeThrough"->False,
                  "Masked"->False,
                  "CompatibilityType"->0,
                  "RotationAngle"->0}], 
                StyleBox["]",
                  FontFamily->"Arial",
                  FontWeight->"Bold"]}],
              FontFamily->"Arial"], 
            StyleBox[
              RowBox[{
                StyleBox[\(KnotLinkReduction . exe\),
                  FontFamily->"Arial"], 
                StyleBox[" ",
                  FontFamily->"Arial"], 
                StyleBox["*",
                  FontFamily->"Arial",
                  FontWeight->"Bold"], 
                StyleBox["2",
                  FontFamily->"Arial",
                  FontWeight->"Bold"]}],
              FontFamily->"Arial"]},
          {
            StyleBox[
              RowBox[{
                StyleBox["UnR",
                  FontFamily->"Arial",
                  FontWeight->"Bold"], 
                StyleBox["[",
                  FontFamily->"Arial",
                  FontWeight->"Bold"], 
                StyleBox["Conway_String",
                  FontFamily->"Arial",
                  FontWeight->"Plain",
                  FontSlant->"Italic",
                  FontTracking->"Plain",
                  FontVariations->{"Underline"->False,
                  "Outline"->False,
                  "Shadow"->False,
                  "StrikeThrough"->False,
                  "Masked"->False,
                  "CompatibilityType"->0,
                  "RotationAngle"->0}], 
                StyleBox["]",
                  FontFamily->"Arial",
                  FontWeight->"Bold"]}],
              FontFamily->"Arial"], 
            StyleBox["None",
              FontFamily->"Arial"]},
          {
            StyleBox[
              RowBox[{
                RowBox[{
                  StyleBox["LinkingNo",
                    FontWeight->"Bold"], 
                  StyleBox["[",
                    FontFamily->"Arial",
                    FontWeight->"Bold"], 
                  StyleBox["Conway_String",
                    FontFamily->"Arial",
                    FontWeight->"Plain",
                    FontSlant->"Italic",
                    FontTracking->"Plain",
                    FontVariations->{"Underline"->False,
                    "Outline"->False,
                    "Shadow"->False,
                    "StrikeThrough"->False,
                    "Masked"->False,
                    "CompatibilityType"->0,
                    "RotationAngle"->0}], 
                  StyleBox["]",
                    FontFamily->"Arial",
                    FontWeight->"Bold"]}], 
                StyleBox["  ",
                  FontFamily->"Arial",
                  FontWeight->"Bold"], 
                StyleBox["or",
                  FontFamily->"Arial",
                  FontVariations->{"CompatibilityType"->0}], 
                StyleBox[" ",
                  FontFamily->"Arial",
                  FontWeight->"Bold"], 
                RowBox[{
                  StyleBox["LinkingNo",
                    FontFamily->"Arial",
                    FontWeight->"Bold"], 
                  StyleBox["[",
                    FontFamily->"Arial",
                    FontWeight->"Bold"], 
                  StyleBox["Dow_List",
                    FontFamily->"Arial",
                    FontSlant->"Italic",
                    FontVariations->{"CompatibilityType"->0}], 
                  StyleBox["]",
                    FontFamily->"Arial",
                    FontWeight->"Bold"]}], 
                StyleBox[" ",
                  FontFamily->"Arial",
                  FontWeight->"Bold"]}],
              FontFamily->"Arial"], 
            StyleBox["None",
              FontFamily->"Arial"]},
          {
            StyleBox[
              RowBox[{
                StyleBox["fCreateGraphics",
                  FontFamily->"Arial",
                  FontWeight->"Bold"], 
                StyleBox["[",
                  FontFamily->"Arial",
                  FontWeight->"Bold"], 
                StyleBox["Conway_String",
                  FontFamily->"Arial",
                  FontWeight->"Plain",
                  FontSlant->"Italic",
                  FontTracking->"Plain",
                  FontVariations->{"Underline"->False,
                  "Outline"->False,
                  "Shadow"->False,
                  "StrikeThrough"->False,
                  "Masked"->False,
                  "CompatibilityType"->0,
                  "RotationAngle"->0}], 
                StyleBox[" ",
                  FontFamily->"Arial",
                  FontWeight->"Plain",
                  FontSlant->"Plain",
                  FontTracking->"Plain",
                  FontVariations->{"Underline"->False,
                  "Outline"->False,
                  "Shadow"->False,
                  "StrikeThrough"->False,
                  "Masked"->False,
                  "CompatibilityType"->0,
                  "RotationAngle"->0}], 
                StyleBox["]",
                  FontFamily->"Arial",
                  FontWeight->"Bold"]}],
              FontFamily->"Arial"], 
            StyleBox[
              RowBox[{
                StyleBox[\(DrawKnot . exe\),
                  FontFamily->"Arial"], 
                StyleBox["*",
                  FontFamily->"Arial",
                  FontWeight->"Bold"], 
                StyleBox["3",
                  FontFamily->"Arial",
                  FontWeight->"Bold"]}],
              FontFamily->"Arial"]}
          },
        GridFrame->True,
        RowLines->True,
        ColumnLines->True]],
    FontFamily->"Courier New"],
  "\n",
  StyleBox["(Notice)",
    FontWeight->"Bold"],
  "\n",
  StyleBox["*1",
    FontWeight->"Bold"],
  "; ",
  StyleBox["fMillett",
    FontWeight->"Bold"],
  " writes a file",
  StyleBox[" ",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["izlaz11.txt",
    FontWeight->"Bold",
    FontVariations->{"CompatibilityType"->0}],
  "  which needs ",
  StyleBox["lmp1.exe as an external program for calculating polynomial \
invariants. ",
    FontVariations->{"CompatibilityType"->0}],
  "Be sure that lmp1.exe is also placed in ",
  StyleBox["K2K",
    FontWeight->"Bold"],
  ".\n",
  StyleBox["*2",
    FontWeight->"Bold"],
  "; ",
  StyleBox["UnKnotLink",
    FontWeight->"Bold"],
  " uses ",
  StyleBox["ReductionKnotLink",
    FontWeight->"Bold"],
  " which needs KnotLinkReduction.exe as an external program.\n",
  StyleBox["*3",
    FontWeight->"Bold"],
  "; Using the external program DrawKnot.exe, ",
  StyleBox["fCreateGraphics ",
    FontWeight->"Bold"],
  "writes a file ",
  StyleBox["c:\\Program Files\\KnotPlot\\graphics.txt",
    FontWeight->"Bold"],
  " that can be opened in ",
  StyleBox["KnotPlot",
    FontWeight->"Bold"],
  " by writing ",
  StyleBox["load graphics.txt",
    FontWeight->"Bold"],
  " in ",
  StyleBox["KnotPlot ",
    FontWeight->"Bold"],
  "command line."
}], "Text",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[0.988296, 0.914076, 0.796887]]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 937}},
CellGrouping->Manual,
WindowSize->{1272, 903},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
StyleDefinitions -> "TutorialBook.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 165, 3, 128, "Title"],
Cell[1944, 58, 166, 2, 49, "Subtitle"],
Cell[2113, 62, 455, 13, 104, "Subsubtitle"],

Cell[CellGroupData[{
Cell[2593, 79, 186, 4, 49, "Subtitle"],

Cell[CellGroupData[{
Cell[2804, 87, 175, 4, 48, "Subsubtitle"],

Cell[CellGroupData[{
Cell[3004, 95, 131, 2, 53, "Subsubsection"],
Cell[3138, 99, 244, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3419, 113, 128, 2, 53, "Subsubsection"],
Cell[3550, 117, 290, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3877, 131, 129, 2, 53, "Subsubsection"],
Cell[4009, 135, 1028, 30, 105, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[5086, 171, 168, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[5279, 179, 131, 2, 53, "Subsubsection"],
Cell[5413, 183, 227, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5677, 197, 128, 2, 53, "Subsubsection"],
Cell[5808, 201, 411, 14, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6256, 220, 129, 2, 53, "Subsubsection"],
Cell[6388, 224, 568, 19, 65, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[7005, 249, 164, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[7194, 257, 131, 2, 53, "Subsubsection"],
Cell[7328, 261, 223, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7588, 275, 128, 2, 53, "Subsubsection"],
Cell[7719, 279, 411, 14, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8167, 298, 129, 2, 53, "Subsubsection"],
Cell[8299, 302, 410, 16, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[8758, 324, 161, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[8944, 332, 131, 2, 53, "Subsubsection"],
Cell[9078, 336, 230, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9345, 350, 128, 2, 53, "Subsubsection"],
Cell[9476, 354, 290, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9803, 368, 129, 2, 53, "Subsubsection"],
Cell[9935, 372, 333, 12, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10305, 389, 134, 2, 53, "Subsubsection"],
Cell[10442, 393, 114, 2, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10593, 400, 123, 2, 53, "Subsubsection"],
Cell[10719, 404, 489, 11, 64, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[11257, 421, 173, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[11455, 429, 131, 2, 53, "Subsubsection"],
Cell[11589, 433, 232, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11858, 447, 128, 2, 53, "Subsubsection"],
Cell[11989, 451, 370, 12, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12396, 468, 134, 2, 53, "Subsubsection"],
Cell[12533, 472, 122, 2, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12692, 479, 129, 2, 53, "Subsubsection"],
Cell[12824, 483, 605, 16, 65, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[13478, 505, 176, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[13679, 513, 131, 2, 53, "Subsubsection"],
Cell[13813, 517, 239, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[14089, 531, 128, 2, 53, "Subsubsection"],
Cell[14220, 535, 280, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[14537, 549, 134, 2, 53, "Subsubsection"],
Cell[14674, 553, 109, 2, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[14820, 560, 129, 2, 53, "Subsubsection"],
Cell[14952, 564, 610, 16, 65, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[15611, 586, 169, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[15805, 594, 131, 2, 53, "Subsubsection"],
Cell[15939, 598, 287, 11, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[16263, 614, 128, 2, 53, "Subsubsection"],
Cell[16394, 618, 938, 32, 64, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[17369, 655, 129, 2, 53, "Subsubsection"],
Cell[17501, 659, 606, 18, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[18144, 682, 134, 2, 53, "Subsubsection"],
Cell[18281, 686, 178, 3, 44, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[18508, 695, 167, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[18700, 703, 131, 2, 53, "Subsubsection"],
Cell[18834, 707, 236, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[19107, 721, 128, 2, 53, "Subsubsection"],
Cell[19238, 725, 290, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[19565, 739, 129, 2, 53, "Subsubsection"],
Cell[19697, 743, 705, 27, 65, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[20451, 776, 167, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[20643, 784, 131, 2, 53, "Subsubsection"],
Cell[20777, 788, 226, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[21040, 802, 128, 2, 53, "Subsubsection"],
Cell[21171, 806, 352, 11, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[21560, 822, 129, 2, 53, "Subsubsection"],
Cell[21692, 826, 306, 10, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[22047, 842, 165, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[22237, 850, 131, 2, 53, "Subsubsection"],
Cell[22371, 854, 234, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[22642, 868, 128, 2, 53, "Subsubsection"],
Cell[22773, 872, 290, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[23100, 886, 129, 2, 53, "Subsubsection"],
Cell[23232, 890, 375, 13, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[23656, 909, 174, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[23855, 917, 131, 2, 53, "Subsubsection"],
Cell[23989, 921, 243, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24269, 935, 128, 2, 53, "Subsubsection"],
Cell[24400, 939, 290, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24727, 953, 129, 2, 53, "Subsubsection"],
Cell[24859, 957, 332, 10, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[25240, 973, 169, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[25434, 981, 131, 2, 53, "Subsubsection"],
Cell[25568, 985, 238, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[25843, 999, 128, 2, 53, "Subsubsection"],
Cell[25974, 1003, 290, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[26301, 1017, 129, 2, 53, "Subsubsection"],
Cell[26433, 1021, 367, 12, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[26849, 1039, 173, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[27047, 1047, 131, 2, 53, "Subsubsection"],
Cell[27181, 1051, 232, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[27450, 1065, 128, 2, 53, "Subsubsection"],
Cell[27581, 1069, 411, 14, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[28029, 1088, 129, 2, 53, "Subsubsection"],
Cell[28161, 1092, 589, 21, 66, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[28799, 1119, 166, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[28990, 1127, 131, 2, 53, "Subsubsection"],
Cell[29124, 1131, 245, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[29406, 1145, 128, 2, 53, "Subsubsection"],
Cell[29537, 1149, 290, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[29864, 1163, 129, 2, 53, "Subsubsection"],
Cell[29996, 1167, 586, 21, 65, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[30631, 1194, 171, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[30827, 1202, 131, 2, 53, "Subsubsection"],
Cell[30961, 1206, 230, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[31228, 1220, 128, 2, 53, "Subsubsection"],
Cell[31359, 1224, 411, 14, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[31807, 1243, 129, 2, 53, "Subsubsection"],
Cell[31939, 1247, 990, 35, 87, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[32978, 1288, 168, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[33171, 1296, 131, 2, 53, "Subsubsection"],
Cell[33305, 1300, 234, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[33576, 1314, 128, 2, 53, "Subsubsection"],
Cell[33707, 1318, 782, 24, 64, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[34526, 1347, 129, 2, 53, "Subsubsection"],
Cell[34658, 1351, 389, 13, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[35096, 1370, 169, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[35290, 1378, 131, 2, 53, "Subsubsection"],
Cell[35424, 1382, 238, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[35699, 1396, 128, 2, 53, "Subsubsection"],
Cell[35830, 1400, 290, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[36157, 1414, 129, 2, 53, "Subsubsection"],
Cell[36289, 1418, 362, 12, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[36700, 1436, 173, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[36898, 1444, 131, 2, 53, "Subsubsection"],
Cell[37032, 1448, 232, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[37301, 1462, 128, 2, 53, "Subsubsection"],
Cell[37432, 1466, 411, 14, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[37880, 1485, 129, 2, 53, "Subsubsection"],
Cell[38012, 1489, 399, 13, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[38460, 1508, 170, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[38655, 1516, 131, 2, 53, "Subsubsection"],
Cell[38789, 1520, 246, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[39072, 1534, 128, 2, 53, "Subsubsection"],
Cell[39203, 1538, 384, 13, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[39624, 1556, 129, 2, 53, "Subsubsection"],
Cell[39756, 1560, 617, 24, 65, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[40422, 1590, 170, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[40617, 1598, 131, 2, 53, "Subsubsection"],
Cell[40751, 1602, 235, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[41023, 1616, 128, 2, 53, "Subsubsection"],
Cell[41154, 1620, 240, 8, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[41431, 1633, 129, 2, 53, "Subsubsection"],
Cell[41563, 1637, 374, 13, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[41986, 1656, 170, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[42181, 1664, 131, 2, 53, "Subsubsection"],
Cell[42315, 1668, 235, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[42587, 1682, 128, 2, 53, "Subsubsection"],
Cell[42718, 1686, 240, 8, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[42995, 1699, 129, 2, 53, "Subsubsection"],
Cell[43127, 1703, 375, 13, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[43551, 1722, 167, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[43743, 1730, 131, 2, 53, "Subsubsection"],
Cell[43877, 1734, 232, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[44146, 1748, 128, 2, 53, "Subsubsection"],
Cell[44277, 1752, 240, 8, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[44554, 1765, 129, 2, 53, "Subsubsection"],
Cell[44686, 1769, 416, 15, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[45151, 1790, 167, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[45343, 1798, 131, 2, 53, "Subsubsection"],
Cell[45477, 1802, 232, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[45746, 1816, 128, 2, 53, "Subsubsection"],
Cell[45877, 1820, 240, 8, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[46154, 1833, 129, 2, 53, "Subsubsection"],
Cell[46286, 1837, 395, 13, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[46730, 1856, 167, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[46922, 1864, 131, 2, 53, "Subsubsection"],
Cell[47056, 1868, 236, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[47329, 1882, 128, 2, 53, "Subsubsection"],
Cell[47460, 1886, 290, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[47787, 1900, 129, 2, 53, "Subsubsection"],
Cell[47919, 1904, 323, 12, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[48291, 1922, 171, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[48487, 1930, 131, 2, 53, "Subsubsection"],
Cell[48621, 1934, 236, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[48894, 1948, 128, 2, 53, "Subsubsection"],
Cell[49025, 1952, 240, 8, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[49302, 1965, 129, 2, 53, "Subsubsection"],
Cell[49434, 1969, 1833, 93, 109, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[51316, 2068, 168, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[51509, 2076, 131, 2, 53, "Subsubsection"],
Cell[51643, 2080, 233, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[51913, 2094, 128, 2, 53, "Subsubsection"],
Cell[52044, 2098, 240, 8, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[52321, 2111, 129, 2, 53, "Subsubsection"],
Cell[52453, 2115, 423, 15, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[52925, 2136, 172, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[53122, 2144, 131, 2, 53, "Subsubsection"],
Cell[53256, 2148, 237, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[53530, 2162, 128, 2, 53, "Subsubsection"],
Cell[53661, 2166, 240, 8, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[53938, 2179, 129, 2, 53, "Subsubsection"],
Cell[54070, 2183, 398, 13, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[54517, 2202, 170, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[54712, 2210, 131, 2, 53, "Subsubsection"],
Cell[54846, 2214, 249, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[55132, 2228, 128, 2, 53, "Subsubsection"],
Cell[55263, 2232, 382, 13, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[55682, 2250, 129, 2, 53, "Subsubsection"],
Cell[55814, 2254, 451, 16, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[56314, 2276, 171, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[56510, 2284, 131, 2, 53, "Subsubsection"],
Cell[56644, 2288, 250, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[56931, 2302, 128, 2, 53, "Subsubsection"],
Cell[57062, 2306, 382, 13, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[57481, 2324, 129, 2, 53, "Subsubsection"],
Cell[57613, 2328, 449, 13, 65, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[58111, 2347, 166, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[58302, 2355, 131, 2, 53, "Subsubsection"],
Cell[58436, 2359, 228, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[58701, 2373, 128, 2, 53, "Subsubsection"],
Cell[58832, 2377, 300, 11, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[59169, 2393, 129, 2, 53, "Subsubsection"],
Cell[59301, 2397, 673, 19, 85, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[60023, 2422, 162, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[60210, 2430, 131, 2, 53, "Subsubsection"],
Cell[60344, 2434, 221, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[60602, 2448, 128, 2, 53, "Subsubsection"],
Cell[60733, 2452, 297, 11, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[61067, 2468, 129, 2, 53, "Subsubsection"],
Cell[61199, 2472, 553, 16, 65, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[61801, 2494, 168, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[61994, 2502, 131, 2, 53, "Subsubsection"],
Cell[62128, 2506, 237, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[62402, 2520, 128, 2, 53, "Subsubsection"],
Cell[62533, 2524, 290, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[62860, 2538, 129, 2, 53, "Subsubsection"],
Cell[62992, 2542, 536, 18, 66, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[63565, 2565, 134, 2, 53, "Subsubsection"],
Cell[63702, 2569, 304, 10, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[64043, 2584, 123, 2, 53, "Subsubsection"],
Cell[64169, 2588, 1053, 27, 64, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[65271, 2621, 161, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[65457, 2629, 131, 2, 53, "Subsubsection"],
Cell[65591, 2633, 230, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[65858, 2647, 128, 2, 53, "Subsubsection"],
Cell[65989, 2651, 290, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[66316, 2665, 129, 2, 53, "Subsubsection"],
Cell[66448, 2669, 404, 12, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[66889, 2686, 134, 2, 53, "Subsubsection"],
Cell[67026, 2690, 114, 2, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[67177, 2697, 123, 2, 53, "Subsubsection"],
Cell[67303, 2701, 834, 22, 64, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[68186, 2729, 168, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[68379, 2737, 131, 2, 53, "Subsubsection"],
Cell[68513, 2741, 237, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[68787, 2755, 128, 2, 53, "Subsubsection"],
Cell[68918, 2759, 290, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[69245, 2773, 129, 2, 53, "Subsubsection"],
Cell[69377, 2777, 360, 12, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[69786, 2795, 172, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[69983, 2803, 131, 2, 53, "Subsubsection"],
Cell[70117, 2807, 231, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[70385, 2821, 128, 2, 53, "Subsubsection"],
Cell[70516, 2825, 370, 12, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[70923, 2842, 129, 2, 53, "Subsubsection"],
Cell[71055, 2846, 491, 18, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[71595, 2870, 165, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[71785, 2878, 131, 2, 53, "Subsubsection"],
Cell[71919, 2882, 228, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[72184, 2896, 128, 2, 53, "Subsubsection"],
Cell[72315, 2900, 248, 8, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[72600, 2913, 129, 2, 53, "Subsubsection"],
Cell[72732, 2917, 348, 10, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[73129, 2933, 167, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[73321, 2941, 131, 2, 53, "Subsubsection"],
Cell[73455, 2945, 236, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[73728, 2959, 128, 2, 53, "Subsubsection"],
Cell[73859, 2963, 290, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[74186, 2977, 129, 2, 53, "Subsubsection"],
Cell[74318, 2981, 462, 15, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[74829, 3002, 172, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[75026, 3010, 131, 2, 53, "Subsubsection"],
Cell[75160, 3014, 231, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[75428, 3028, 128, 2, 53, "Subsubsection"],
Cell[75559, 3032, 370, 12, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[75966, 3049, 129, 2, 53, "Subsubsection"],
Cell[76098, 3053, 818, 26, 86, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[76965, 3085, 165, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[77155, 3093, 131, 2, 53, "Subsubsection"],
Cell[77289, 3097, 228, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[77554, 3111, 128, 2, 53, "Subsubsection"],
Cell[77685, 3115, 315, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[78037, 3129, 129, 2, 53, "Subsubsection"],
Cell[78169, 3133, 896, 24, 85, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[79114, 3163, 169, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[79308, 3171, 131, 2, 53, "Subsubsection"],
Cell[79442, 3175, 232, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[79711, 3189, 128, 2, 53, "Subsubsection"],
Cell[79842, 3193, 285, 8, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[80164, 3206, 129, 2, 53, "Subsubsection"],
Cell[80296, 3210, 829, 22, 85, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[81174, 3238, 167, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[81366, 3246, 131, 2, 53, "Subsubsection"],
Cell[81500, 3250, 246, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[81783, 3264, 128, 2, 53, "Subsubsection"],
Cell[81914, 3268, 454, 14, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[82405, 3287, 129, 2, 53, "Subsubsection"],
Cell[82537, 3291, 359, 12, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[82945, 3309, 168, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[83138, 3317, 131, 2, 53, "Subsubsection"],
Cell[83272, 3321, 231, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[83540, 3335, 128, 2, 53, "Subsubsection"],
Cell[83671, 3339, 374, 12, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[84082, 3356, 129, 2, 53, "Subsubsection"],
Cell[84214, 3360, 392, 15, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[84655, 3381, 166, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[84846, 3389, 131, 2, 53, "Subsubsection"],
Cell[84980, 3393, 225, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[85242, 3407, 128, 2, 53, "Subsubsection"],
Cell[85373, 3411, 229, 8, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[85639, 3424, 129, 2, 53, "Subsubsection"],
Cell[85771, 3428, 336, 12, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[86156, 3446, 168, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[86349, 3454, 131, 2, 53, "Subsubsection"],
Cell[86483, 3458, 237, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[86757, 3472, 128, 2, 53, "Subsubsection"],
Cell[86888, 3476, 403, 14, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[87328, 3495, 129, 2, 53, "Subsubsection"],
Cell[87460, 3499, 542, 18, 65, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[88051, 3523, 166, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[88242, 3531, 131, 2, 53, "Subsubsection"],
Cell[88376, 3535, 237, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[88650, 3549, 128, 2, 53, "Subsubsection"],
Cell[88781, 3553, 351, 11, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[89169, 3569, 129, 2, 53, "Subsubsection"],
Cell[89301, 3573, 1025, 26, 106, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[90375, 3605, 170, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[90570, 3613, 131, 2, 53, "Subsubsection"],
Cell[90704, 3617, 229, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[90970, 3631, 128, 2, 53, "Subsubsection"],
Cell[91101, 3635, 370, 12, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[91508, 3652, 129, 2, 53, "Subsubsection"],
Cell[91640, 3656, 414, 15, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[92103, 3677, 172, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[92300, 3685, 131, 2, 53, "Subsubsection"],
Cell[92434, 3689, 240, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[92711, 3703, 128, 2, 53, "Subsubsection"],
Cell[92842, 3707, 294, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[93173, 3721, 129, 2, 53, "Subsubsection"],
Cell[93305, 3725, 464, 13, 65, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[93818, 3744, 173, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[94016, 3752, 131, 2, 53, "Subsubsection"],
Cell[94150, 3756, 242, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[94429, 3770, 128, 2, 53, "Subsubsection"],
Cell[94560, 3774, 339, 12, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[94936, 3791, 129, 2, 53, "Subsubsection"],
Cell[95068, 3795, 768, 25, 86, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[95873, 3825, 134, 2, 53, "Subsubsection"],
Cell[96010, 3829, 121, 2, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[96168, 3836, 123, 2, 53, "Subsubsection"],
Cell[96294, 3840, 831, 26, 87, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[97174, 3872, 170, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[97369, 3880, 131, 2, 53, "Subsubsection"],
Cell[97503, 3884, 239, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[97779, 3898, 128, 2, 53, "Subsubsection"],
Cell[97910, 3902, 290, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[98237, 3916, 129, 2, 53, "Subsubsection"],
Cell[98369, 3920, 401, 15, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[98807, 3940, 123, 2, 53, "Subsubsection"],
Cell[98933, 3944, 372, 12, 44, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[99354, 3962, 169, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[99548, 3970, 131, 2, 53, "Subsubsection"],
Cell[99682, 3974, 228, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[99947, 3988, 128, 2, 53, "Subsubsection"],
Cell[100078, 3992, 370, 12, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[100485, 4009, 129, 2, 53, "Subsubsection"],
Cell[100617, 4013, 644, 17, 85, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[101310, 4036, 169, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[101504, 4044, 131, 2, 53, "Subsubsection"],
Cell[101638, 4048, 238, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[101913, 4062, 128, 2, 53, "Subsubsection"],
Cell[102044, 4066, 290, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[102371, 4080, 129, 2, 53, "Subsubsection"],
Cell[102503, 4084, 359, 12, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[102911, 4102, 179, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[103115, 4110, 131, 2, 53, "Subsubsection"],
Cell[103249, 4114, 248, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[103534, 4128, 128, 2, 53, "Subsubsection"],
Cell[103665, 4132, 290, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[103992, 4146, 129, 2, 53, "Subsubsection"],
Cell[104124, 4150, 419, 13, 65, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[104592, 4169, 166, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[104783, 4177, 131, 2, 53, "Subsubsection"],
Cell[104917, 4181, 241, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[105195, 4195, 128, 2, 53, "Subsubsection"],
Cell[105326, 4199, 382, 14, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[105745, 4218, 129, 2, 53, "Subsubsection"],
Cell[105877, 4222, 531, 16, 65, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[106457, 4244, 168, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[106650, 4252, 131, 2, 53, "Subsubsection"],
Cell[106784, 4256, 234, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[107055, 4270, 128, 2, 53, "Subsubsection"],
Cell[107186, 4274, 283, 10, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[107506, 4289, 129, 2, 53, "Subsubsection"],
Cell[107638, 4293, 267, 9, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[107954, 4308, 166, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[108145, 4316, 131, 2, 53, "Subsubsection"],
Cell[108279, 4320, 235, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[108551, 4334, 128, 2, 53, "Subsubsection"],
Cell[108682, 4338, 294, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[109013, 4352, 129, 2, 53, "Subsubsection"],
Cell[109145, 4356, 294, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[109476, 4370, 134, 2, 53, "Subsubsection"],
Cell[109613, 4374, 114, 2, 44, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[109776, 4382, 168, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[109969, 4390, 131, 2, 53, "Subsubsection"],
Cell[110103, 4394, 234, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[110374, 4408, 128, 2, 53, "Subsubsection"],
Cell[110505, 4412, 452, 11, 64, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[110994, 4428, 129, 2, 53, "Subsubsection"],
Cell[111126, 4432, 364, 15, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[111527, 4452, 134, 2, 53, "Subsubsection"],
Cell[111664, 4456, 114, 2, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[111815, 4463, 123, 2, 53, "Subsubsection"],
Cell[111941, 4467, 489, 11, 84, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[112479, 4484, 166, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[112670, 4492, 131, 2, 53, "Subsubsection"],
Cell[112804, 4496, 229, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[113070, 4510, 128, 2, 53, "Subsubsection"],
Cell[113201, 4514, 317, 11, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[113555, 4530, 129, 2, 53, "Subsubsection"],
Cell[113687, 4534, 383, 12, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[114119, 4552, 166, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[114310, 4560, 131, 2, 53, "Subsubsection"],
Cell[114444, 4564, 225, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[114706, 4578, 128, 2, 53, "Subsubsection"],
Cell[114837, 4582, 370, 12, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[115244, 4599, 129, 2, 53, "Subsubsection"],
Cell[115376, 4603, 437, 15, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[115862, 4624, 162, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[116049, 4632, 131, 2, 53, "Subsubsection"],
Cell[116183, 4636, 231, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[116451, 4650, 128, 2, 53, "Subsubsection"],
Cell[116582, 4654, 290, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[116909, 4668, 129, 2, 53, "Subsubsection"],
Cell[117041, 4672, 477, 16, 65, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[117567, 4694, 165, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[117757, 4702, 131, 2, 53, "Subsubsection"],
Cell[117891, 4706, 234, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[118162, 4720, 128, 2, 53, "Subsubsection"],
Cell[118293, 4724, 290, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[118620, 4738, 129, 2, 53, "Subsubsection"],
Cell[118752, 4742, 503, 16, 65, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[119304, 4764, 172, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[119501, 4772, 131, 2, 53, "Subsubsection"],
Cell[119635, 4776, 233, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[119905, 4790, 128, 2, 53, "Subsubsection"],
Cell[120036, 4794, 372, 12, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[120445, 4811, 129, 2, 53, "Subsubsection"],
Cell[120577, 4815, 387, 13, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[121013, 4834, 166, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[121204, 4842, 131, 2, 53, "Subsubsection"],
Cell[121338, 4846, 235, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[121610, 4860, 128, 2, 53, "Subsubsection"],
Cell[121741, 4864, 290, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[122068, 4878, 129, 2, 53, "Subsubsection"],
Cell[122200, 4882, 354, 12, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[122603, 4900, 173, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[122801, 4908, 131, 2, 53, "Subsubsection"],
Cell[122935, 4912, 242, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[123214, 4926, 128, 2, 53, "Subsubsection"],
Cell[123345, 4930, 290, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[123672, 4944, 129, 2, 53, "Subsubsection"],
Cell[123804, 4948, 403, 15, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[124256, 4969, 169, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[124450, 4977, 131, 2, 53, "Subsubsection"],
Cell[124584, 4981, 228, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[124849, 4995, 128, 2, 53, "Subsubsection"],
Cell[124980, 4999, 370, 12, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[125387, 5016, 129, 2, 53, "Subsubsection"],
Cell[125519, 5020, 783, 24, 86, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[126351, 5050, 167, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[126543, 5058, 131, 2, 53, "Subsubsection"],
Cell[126677, 5062, 226, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[126940, 5076, 128, 2, 53, "Subsubsection"],
Cell[127071, 5080, 370, 12, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[127478, 5097, 129, 2, 53, "Subsubsection"],
Cell[127610, 5101, 528, 18, 65, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[128187, 5125, 166, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[128378, 5133, 131, 2, 53, "Subsubsection"],
Cell[128512, 5137, 227, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[128776, 5151, 128, 2, 53, "Subsubsection"],
Cell[128907, 5155, 372, 12, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[129316, 5172, 129, 2, 53, "Subsubsection"],
Cell[129448, 5176, 464, 15, 65, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[129961, 5197, 169, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[130155, 5205, 131, 2, 53, "Subsubsection"],
Cell[130289, 5209, 238, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[130564, 5223, 128, 2, 53, "Subsubsection"],
Cell[130695, 5227, 290, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[131022, 5241, 129, 2, 53, "Subsubsection"],
Cell[131154, 5245, 380, 12, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[131583, 5263, 170, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[131778, 5271, 131, 2, 53, "Subsubsection"],
Cell[131912, 5275, 232, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[132181, 5289, 128, 2, 53, "Subsubsection"],
Cell[132312, 5293, 288, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[132637, 5307, 129, 2, 53, "Subsubsection"],
Cell[132769, 5311, 464, 18, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[133282, 5335, 168, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[133475, 5343, 131, 2, 53, "Subsubsection"],
Cell[133609, 5347, 235, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[133881, 5361, 128, 2, 53, "Subsubsection"],
Cell[134012, 5365, 335, 11, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[134384, 5381, 129, 2, 53, "Subsubsection"],
Cell[134516, 5385, 440, 15, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[135005, 5406, 171, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[135201, 5414, 131, 2, 53, "Subsubsection"],
Cell[135335, 5418, 237, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[135609, 5432, 128, 2, 53, "Subsubsection"],
Cell[135740, 5436, 287, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[136064, 5450, 129, 2, 53, "Subsubsection"],
Cell[136196, 5454, 625, 21, 65, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[136870, 5481, 171, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[137066, 5489, 131, 2, 53, "Subsubsection"],
Cell[137200, 5493, 233, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[137470, 5507, 128, 2, 53, "Subsubsection"],
Cell[137601, 5511, 341, 11, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[137979, 5527, 129, 2, 53, "Subsubsection"],
Cell[138111, 5531, 395, 12, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[138555, 5549, 166, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[138746, 5557, 131, 2, 53, "Subsubsection"],
Cell[138880, 5561, 235, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[139152, 5575, 128, 2, 53, "Subsubsection"],
Cell[139283, 5579, 290, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[139610, 5593, 129, 2, 53, "Subsubsection"],
Cell[139742, 5597, 650, 19, 66, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[140429, 5621, 134, 2, 53, "Subsubsection"],
Cell[140566, 5625, 327, 10, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[140930, 5640, 123, 2, 53, "Subsubsection"],
Cell[141056, 5644, 780, 27, 108, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[141885, 5677, 171, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[142081, 5685, 131, 2, 53, "Subsubsection"],
Cell[142215, 5689, 230, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[142482, 5703, 128, 2, 53, "Subsubsection"],
Cell[142613, 5707, 370, 12, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[143020, 5724, 129, 2, 53, "Subsubsection"],
Cell[143152, 5728, 671, 17, 85, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[143872, 5751, 171, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[144068, 5759, 131, 2, 53, "Subsubsection"],
Cell[144202, 5763, 235, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[144474, 5777, 128, 2, 53, "Subsubsection"],
Cell[144605, 5781, 555, 18, 64, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[145197, 5804, 129, 2, 53, "Subsubsection"],
Cell[145329, 5808, 558, 19, 65, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[145936, 5833, 174, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[146135, 5841, 131, 2, 53, "Subsubsection"],
Cell[146269, 5845, 238, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[146544, 5859, 128, 2, 53, "Subsubsection"],
Cell[146675, 5863, 232, 6, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[146944, 5874, 129, 2, 53, "Subsubsection"],
Cell[147076, 5878, 333, 12, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[147458, 5896, 168, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[147651, 5904, 131, 2, 53, "Subsubsection"],
Cell[147785, 5908, 227, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[148049, 5922, 128, 2, 53, "Subsubsection"],
Cell[148180, 5926, 370, 12, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[148587, 5943, 134, 2, 53, "Subsubsection"],
Cell[148724, 5947, 122, 2, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[148883, 5954, 129, 2, 53, "Subsubsection"],
Cell[149015, 5958, 768, 24, 65, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[149832, 5988, 173, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[150030, 5996, 131, 2, 53, "Subsubsection"],
Cell[150164, 6000, 232, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[150433, 6014, 128, 2, 53, "Subsubsection"],
Cell[150564, 6018, 290, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[150891, 6032, 134, 2, 53, "Subsubsection"],
Cell[151028, 6036, 122, 2, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[151187, 6043, 129, 2, 53, "Subsubsection"],
Cell[151319, 6047, 653, 16, 85, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[152021, 6069, 168, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[152214, 6077, 131, 2, 53, "Subsubsection"],
Cell[152348, 6081, 237, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[152622, 6095, 128, 2, 53, "Subsubsection"],
Cell[152753, 6099, 294, 9, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[153084, 6113, 129, 2, 53, "Subsubsection"],
Cell[153216, 6117, 457, 13, 65, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[153722, 6136, 169, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[153916, 6144, 131, 2, 53, "Subsubsection"],
Cell[154050, 6148, 232, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[154319, 6162, 128, 2, 53, "Subsubsection"],
Cell[154450, 6166, 273, 8, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[154760, 6179, 129, 2, 53, "Subsubsection"],
Cell[154892, 6183, 552, 16, 65, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[155493, 6205, 166, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[155684, 6213, 131, 2, 53, "Subsubsection"],
Cell[155818, 6217, 225, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[156080, 6231, 128, 2, 53, "Subsubsection"],
Cell[156211, 6235, 314, 11, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[156562, 6251, 129, 2, 53, "Subsubsection"],
Cell[156694, 6255, 681, 27, 65, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[157424, 6288, 170, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[157619, 6296, 131, 2, 53, "Subsubsection"],
Cell[157753, 6300, 236, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[158026, 6314, 128, 2, 53, "Subsubsection"],
Cell[158157, 6318, 328, 11, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[158522, 6334, 129, 2, 53, "Subsubsection"],
Cell[158654, 6338, 365, 12, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[159068, 6356, 166, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[159259, 6364, 131, 2, 53, "Subsubsection"],
Cell[159393, 6368, 225, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[159655, 6382, 128, 2, 53, "Subsubsection"],
Cell[159786, 6386, 411, 14, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[160234, 6405, 129, 2, 53, "Subsubsection"],
Cell[160366, 6409, 560, 21, 65, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[160975, 6436, 165, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[161165, 6444, 131, 2, 53, "Subsubsection"],
Cell[161299, 6448, 224, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[161560, 6462, 128, 2, 53, "Subsubsection"],
Cell[161691, 6466, 411, 14, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[162139, 6485, 129, 2, 53, "Subsubsection"],
Cell[162271, 6489, 543, 21, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[162863, 6516, 166, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[163054, 6524, 131, 2, 53, "Subsubsection"],
Cell[163188, 6528, 230, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[163455, 6542, 128, 2, 53, "Subsubsection"],
Cell[163586, 6546, 555, 18, 64, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[164178, 6569, 129, 2, 53, "Subsubsection"],
Cell[164310, 6573, 774, 26, 85, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[165133, 6605, 171, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[165329, 6613, 131, 2, 53, "Subsubsection"],
Cell[165463, 6617, 236, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[165736, 6631, 128, 2, 53, "Subsubsection"],
Cell[165867, 6635, 240, 8, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[166144, 6648, 129, 2, 53, "Subsubsection"],
Cell[166276, 6652, 431, 15, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[166756, 6673, 166, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[166947, 6681, 131, 2, 53, "Subsubsection"],
Cell[167081, 6685, 231, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[167349, 6699, 128, 2, 53, "Subsubsection"],
Cell[167480, 6703, 240, 8, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[167757, 6716, 129, 2, 53, "Subsubsection"],
Cell[167889, 6720, 420, 15, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[168358, 6741, 170, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[168553, 6749, 131, 2, 53, "Subsubsection"],
Cell[168687, 6753, 235, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[168959, 6767, 128, 2, 53, "Subsubsection"],
Cell[169090, 6771, 240, 8, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[169367, 6784, 129, 2, 53, "Subsubsection"],
Cell[169499, 6788, 436, 15, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[169984, 6809, 170, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[170179, 6817, 131, 2, 53, "Subsubsection"],
Cell[170313, 6821, 235, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[170585, 6835, 128, 2, 53, "Subsubsection"],
Cell[170716, 6839, 240, 8, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[170993, 6852, 129, 2, 53, "Subsubsection"],
Cell[171125, 6856, 442, 16, 45, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[171616, 6878, 166, 4, 40, "Subsubtitle"],

Cell[CellGroupData[{
Cell[171807, 6886, 131, 2, 53, "Subsubsection"],
Cell[171941, 6890, 241, 9, 45, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[172219, 6904, 128, 2, 53, "Subsubsection"],
Cell[172350, 6908, 417, 17, 44, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[172804, 6930, 129, 2, 53, "Subsubsection"],
Cell[172936, 6934, 708, 23, 65, "Text"]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[173705, 6964, 134, 3, 49, "Subtitle"],

Cell[CellGroupData[{
Cell[173864, 6971, 128, 3, 49, "Subtitle"],
Cell[173995, 6976, 15106, 404, 821, "Text"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[189150, 7386, 182, 5, 49, "Subtitle"],
Cell[189335, 7393, 14497, 412, 455, "Text"]
}, Closed]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

